# Copyright (c) 2012-2014, Michael DeHaan <michael.dehaan@gmail.com>
# Copyright (c) 2017, Toshio Kuratomi <tkuraotmi@ansible.com>
# Copyright (c) 2020, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import copy
import traceback

from ansible.config.manager import ensure_type
from ansible.errors import AnsibleError, AnsibleFileNotFound, AnsibleAction, AnsibleActionFail
from ansible.module_utils.parsing.convert_bool import boolean
from ansible.module_utils.six import string_types
from ansible.module_utils._text import to_text, to_bytes, to_native
from ansible.plugins.action import ActionBase


class ActionModule(ActionBase):

    TRANSFERS_FILES = True
    DEFAULT_NEWLINE_SEQUENCE = "\n"

    def _ensure_invocation(self, result):
        # NOTE: adding invocation arguments here needs to be kept in sync with
        # any no_log specified in the argument_spec in the module.
        if 'invocation' not in result:
            if self._play_context.no_log:
                result['invocation'] = "CENSORED: no_log is set"
            else:
                result['invocation'] = self._task.args.copy()
                result['invocation']['module_args'] = self._task.args.copy()

        return result

    def run(self, tmp=None, task_vars=None):
        ''' handler for k8s options '''
        if task_vars is None:
            task_vars = dict()

        result = super(ActionModule, self).run(tmp, task_vars)
        del tmp  # tmp no longer has any effect

        new_module_args = copy.deepcopy(self._task.args)
        kubeconfig = self._task.args.get('kubeconfig', None)
        # find the file in the expected search path
        if kubeconfig:
            try:
                # find in expected paths
                kubeconfig = self._find_needle('files', kubeconfig)
            except AnsibleError as e:
                result['failed'] = True
                result['msg'] = to_text(e)
                result['exception'] = traceback.format_exc()
                return result

        if kubeconfig:
            # decrypt kubeconfig found
            actual_file = self._loader.get_real_file(kubeconfig, decrypt=True)
            new_module_args['kubeconfig'] = actual_file

        # find the file in the expected search path
        src = self._task.args.get('src', None)
        if src:
            try:
                # find in expected paths
                src = self._find_needle('files', src)
            except AnsibleError as e:
                result['failed'] = True
                result['msg'] = to_text(e)
                result['exception'] = traceback.format_exc()
                return result

        if src:
            new_module_args['src'] = src

        template = self._task.args.get('template', None)
        if template:
            # template is only supported by k8s module.
            if self._task.action not in ('k8s', 'community.kubernetes.k8s', 'community.okd.k8s'):
                raise AnsibleActionFail("'template' is only supported parameter for 'k8s' module.")
            if isinstance(template, string_types):
                # treat this as raw_params
                template_path = template
                newline_sequence = self.DEFAULT_NEWLINE_SEQUENCE
                variable_start_string = None
                variable_end_string = None
                block_start_string = None
                block_end_string = None
                trim_blocks = True
                lstrip_blocks = False
            elif isinstance(template, dict):
                template_args = template
                template_path = template_args.get('path', None)
                if not template:
                    raise AnsibleActionFail("Please specify path for template.")

                # Options type validation strings
                for s_type in ('newline_sequence', 'variable_start_string', 'variable_end_string', 'block_start_string',
                               'block_end_string'):
                    if s_type in template_args:
                        value = ensure_type(template_args[s_type], 'string')
                        if value is not None and not isinstance(value, string_types):
                            raise AnsibleActionFail("%s is expected to be a string, but got %s instead" % (s_type, type(value)))
                try:
                    trim_blocks = boolean(template_args.get('trim_blocks', True), strict=False)
                    lstrip_blocks = boolean(template_args.get('lstrip_blocks', False), strict=False)
                except TypeError as e:
                    raise AnsibleActionFail(to_native(e))

                newline_sequence = template_args.get('newline_sequence', self.DEFAULT_NEWLINE_SEQUENCE)
                variable_start_string = template_args.get('variable_start_string', None)
                variable_end_string = template_args.get('variable_end_string', None)
                block_start_string = template_args.get('block_start_string', None)
                block_end_string = template_args.get('block_end_string', None)
            else:
                raise AnsibleActionFail("Error while reading template file - "
                                        "a string or dict for template expected, but got %s instead" % type(template))
            try:
                source = self._find_needle('templates', template_path)
            except AnsibleError as e:
                raise AnsibleActionFail(to_text(e))

            # Option `lstrip_blocks' was added in Jinja2 version 2.7.
            if lstrip_blocks:
                try:
                    import jinja2.defaults
                except ImportError:
                    raise AnsibleError('Unable to import Jinja2 defaults for determining Jinja2 features.')

                try:
                    jinja2.defaults.LSTRIP_BLOCKS
                except AttributeError:
                    raise AnsibleError("Option `lstrip_blocks' is only available in Jinja2 versions >=2.7")

            wrong_sequences = ["\\n", "\\r", "\\r\\n"]
            allowed_sequences = ["\n", "\r", "\r\n"]

            # We need to convert unescaped sequences to proper escaped sequences for Jinja2
            if newline_sequence in wrong_sequences:
                newline_sequence = allowed_sequences[wrong_sequences.index(newline_sequence)]
            elif newline_sequence not in allowed_sequences:
                raise AnsibleActionFail("newline_sequence needs to be one of: \n, \r or \r\n")

            # Get vault decrypted tmp file
            try:
                tmp_source = self._loader.get_real_file(source)
            except AnsibleFileNotFound as e:
                raise AnsibleActionFail("could not find template=%s, %s" % (source, to_text(e)))
            b_tmp_source = to_bytes(tmp_source, errors='surrogate_or_strict')

            # template the source data locally & get ready to transfer
            try:
                with open(b_tmp_source, 'rb') as f:
                    try:
                        template_data = to_text(f.read(), errors='surrogate_or_strict')
                    except UnicodeError:
                        raise AnsibleActionFail("Template source files must be utf-8 encoded")

                # add ansible 'template' vars
                temp_vars = task_vars.copy()
                old_vars = self._templar.available_variables

                self._templar.environment.newline_sequence = newline_sequence
                if block_start_string is not None:
                    self._templar.environment.block_start_string = block_start_string
                if block_end_string is not None:
                    self._templar.environment.block_end_string = block_end_string
                if variable_start_string is not None:
                    self._templar.environment.variable_start_string = variable_start_string
                if variable_end_string is not None:
                    self._templar.environment.variable_end_string = variable_end_string
                self._templar.environment.trim_blocks = trim_blocks
                self._templar.environment.lstrip_blocks = lstrip_blocks
                self._templar.available_variables = temp_vars
                resultant = self._templar.do_template(template_data, preserve_trailing_newlines=True, escape_backslashes=False)
                self._templar.available_variables = old_vars
                resource_definition = self._task.args.get('definition', None)
                if not resource_definition:
                    new_module_args.pop('template')
                new_module_args['definition'] = resultant
            except AnsibleAction:
                raise
            except Exception as e:
                raise AnsibleActionFail("%s: %s" % (type(e).__name__, to_text(e)))
            finally:
                self._loader.cleanup_tmp_file(b_tmp_source)

        # Execute the k8s_* module.
        module_return = self._execute_module(module_name=self._task.action, module_args=new_module_args, task_vars=task_vars)

        # Delete tmp path
        self._remove_tmp_path(self._connection._shell.tmpdir)

        result.update(module_return)

        return self._ensure_invocation(result)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2019 Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: hetzner_failover_ip
short_description: Manage Hetzner's failover IPs
author:
  - Felix Fontein (@felixfontein)
description:
  - Manage Hetzner's failover IPs.
seealso:
  - name: Failover IP documentation
    description: Hetzner's documentation on failover IPs.
    link: https://wiki.hetzner.de/index.php/Failover/en
  - module: community.general.hetzner_failover_ip_info
    description: Retrieve information on failover IPs.
extends_documentation_fragment:
- community.general.hetzner

options:
  failover_ip:
    description: The failover IP address.
    type: str
    required: yes
  state:
    description:
      - Defines whether the IP will be routed or not.
      - If set to C(routed), I(value) must be specified.
    type: str
    choices:
      - routed
      - unrouted
    default: routed
  value:
    description:
      - The new value for the failover IP address.
      - Required when setting I(state) to C(routed).
    type: str
  timeout:
    description:
      - Timeout to use when routing or unrouting the failover IP.
      - Note that the API call returns when the failover IP has been
        successfully routed to the new address, respectively successfully
        unrouted.
    type: int
    default: 180
'''

EXAMPLES = r'''
- name: Set value of failover IP 1.2.3.4 to 5.6.7.8
  community.general.hetzner_failover_ip:
    hetzner_user: foo
    hetzner_password: bar
    failover_ip: 1.2.3.4
    value: 5.6.7.8

- name: Set value of failover IP 1.2.3.4 to unrouted
  community.general.hetzner_failover_ip:
    hetzner_user: foo
    hetzner_password: bar
    failover_ip: 1.2.3.4
    state: unrouted
'''

RETURN = r'''
value:
  description:
    - The value of the failover IP.
    - Will be C(none) if the IP is unrouted.
  returned: success
  type: str
state:
  description:
    - Will be C(routed) or C(unrouted).
  returned: success
  type: str
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.general.plugins.module_utils.hetzner import (
    HETZNER_DEFAULT_ARGUMENT_SPEC,
    get_failover,
    set_failover,
    get_failover_state,
)


def main():
    argument_spec = dict(
        failover_ip=dict(type='str', required=True),
        state=dict(type='str', default='routed', choices=['routed', 'unrouted']),
        value=dict(type='str'),
        timeout=dict(type='int', default=180),
    )
    argument_spec.update(HETZNER_DEFAULT_ARGUMENT_SPEC)
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=(
            ('state', 'routed', ['value']),
        ),
    )

    failover_ip = module.params['failover_ip']
    value = get_failover(module, failover_ip)
    changed = False
    before = get_failover_state(value)

    if module.params['state'] == 'routed':
        new_value = module.params['value']
    else:
        new_value = None

    if value != new_value:
        if module.check_mode:
            value = new_value
            changed = True
        else:
            value, changed = set_failover(module, failover_ip, new_value, timeout=module.params['timeout'])

    after = get_failover_state(value)
    module.exit_json(
        changed=changed,
        diff=dict(
            before=before,
            after=after,
        ),
        **after
    )


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Copyright 2018 Luke Murphy <lukewm@riseup.net>
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import pytest
import sys

linode_apiv4 = pytest.importorskip('linode_api4')
mandatory_py_version = pytest.mark.skipif(
    sys.version_info < (2, 7),
    reason='The linode_api4 dependency requires python2.7 or higher'
)


from ansible.errors import AnsibleError, AnsibleParserError
from ansible_collections.community.general.plugins.inventory.linode import InventoryModule


@pytest.fixture(scope="module")
def inventory():
    return InventoryModule()


def test_access_token_lookup(inventory):
    inventory._options = {'access_token': None}
    with pytest.raises(AnsibleError) as error_message:
        inventory._build_client()
        assert 'Could not retrieve Linode access token' in error_message


def test_validate_option(inventory):
    assert ['eu-west'] == inventory._validate_option('regions', list, 'eu-west')
    assert ['eu-west'] == inventory._validate_option('regions', list, ['eu-west'])


def test_validation_option_bad_option(inventory):
    with pytest.raises(AnsibleParserError) as error_message:
        inventory._validate_option('regions', dict, [])
        assert "The option filters ([]) must be a <class 'dict'>" == error_message


def test_empty_config_query_options(inventory):
    regions, types = inventory._get_query_options({})
    assert regions == types == []


def test_conig_query_options(inventory):
    regions, types = inventory._get_query_options({
        'regions': ['eu-west', 'us-east'],
        'types': ['g5-standard-2', 'g6-standard-2'],
    })

    assert regions == ['eu-west', 'us-east']
    assert types == ['g5-standard-2', 'g6-standard-2']


def test_verify_file_bad_config(inventory):
    assert inventory.verify_file('foobar.linde.yml') is False



# -*- coding: utf-8 -*-
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
    name: grafana_annotations
    type: notification
    short_description: send ansible events as annotations on charts to grafana over http api.
    author: "Rémi REY (@rrey)"
    description:
      - This callback will report start, failed and stats events to Grafana as annotations (https://grafana.com)
    requirements:
      - whitelisting in configuration
    options:
      grafana_url:
        description: Grafana annotations api URL
        required: True
        env:
          - name: GRAFANA_URL
        ini:
          - section: callback_grafana_annotations
            key: grafana_url
        type: string
      validate_certs:
        description: validate the SSL certificate of the Grafana server. (For HTTPS url)
        env:
          - name: GRAFANA_VALIDATE_CERT
        ini:
          - section: callback_grafana_annotations
            key: validate_grafana_certs
          - section: callback_grafana_annotations
            key: validate_certs
        default: True
        type: bool
        aliases: [ validate_grafana_certs ]
      http_agent:
        description: The HTTP 'User-agent' value to set in HTTP requets.
        env:
          - name: HTTP_AGENT
        ini:
          - section: callback_grafana_annotations
            key: http_agent
        default: 'Ansible (grafana_annotations callback)'
        type: string
      grafana_api_key:
        description: Grafana API key, allowing to authenticate when posting on the HTTP API.
                     If not provided, grafana_login and grafana_password will
                     be required.
        env:
          - name: GRAFANA_API_KEY
        ini:
          - section: callback_grafana_annotations
            key: grafana_api_key
        type: string
      grafana_user:
        description: Grafana user used for authentication. Ignored if grafana_api_key is provided.
        env:
          - name: GRAFANA_USER
        ini:
          - section: callback_grafana_annotations
            key: grafana_user
        default: ansible
        type: string
      grafana_password:
        description: Grafana password used for authentication. Ignored if grafana_api_key is provided.
        env:
          - name: GRAFANA_PASSWORD
        ini:
          - section: callback_grafana_annotations
            key: grafana_password
        default: ansible
        type: string
      grafana_dashboard_id:
        description: The grafana dashboard id where the annotation shall be created.
        env:
          - name: GRAFANA_DASHBOARD_ID
        ini:
          - section: callback_grafana_annotations
            key: grafana_dashboard_id
        type: integer
      grafana_panel_ids:
        description: The grafana panel ids where the annotation shall be created.
                     Give a single integer or a comma-separated list of integers.
        env:
          - name: GRAFANA_PANEL_IDS
        ini:
          - section: callback_grafana_annotations
            key: grafana_panel_ids
        default: []
        type: list
'''

import json
import socket
import getpass
from datetime import datetime

from ansible.module_utils._text import to_text
from ansible.module_utils.urls import open_url
from ansible.plugins.callback import CallbackBase


PLAYBOOK_START_TXT = """\
Started playbook {playbook}

From '{hostname}'
By user '{username}'
"""

PLAYBOOK_ERROR_TXT = """\
Playbook {playbook} Failure !

From '{hostname}'
By user '{username}'

'{task}' failed on {host}

debug: {result}
"""

PLAYBOOK_STATS_TXT = """\
Playbook {playbook}
Duration: {duration}
Status: {status}

From '{hostname}'
By user '{username}'

Result:
{summary}
"""


def to_millis(dt):
    return int(dt.strftime('%s')) * 1000


class CallbackModule(CallbackBase):
    """
    ansible grafana callback plugin
    ansible.cfg:
        callback_plugins   = <path_to_callback_plugins_folder>
        callback_whitelist = grafana_annotations
    and put the plugin in <path_to_callback_plugins_folder>
    """

    CALLBACK_VERSION = 2.0
    CALLBACK_TYPE = 'aggregate'
    CALLBACK_NAME = 'community.grafana.grafana_annotations'
    CALLBACK_NEEDS_WHITELIST = True

    def __init__(self, display=None):

        super(CallbackModule, self).__init__(display=display)

        self.headers = {'Content-Type': 'application/json'}
        self.force_basic_auth = False
        self.hostname = socket.gethostname()
        self.username = getpass.getuser()
        self.start_time = datetime.now()
        self.errors = 0

    def set_options(self, task_keys=None, var_options=None, direct=None):

        super(CallbackModule, self).set_options(task_keys=task_keys, var_options=var_options, direct=direct)

        self.grafana_api_key = self.get_option('grafana_api_key')
        self.grafana_url = self.get_option('grafana_url')
        self.validate_grafana_certs = self.get_option('validate_certs')
        self.http_agent = self.get_option('http_agent')
        self.grafana_user = self.get_option('grafana_user')
        self.grafana_password = self.get_option('grafana_password')
        self.dashboard_id = self.get_option('grafana_dashboard_id')
        self.panel_ids = self.get_option('grafana_panel_ids')

        if self.grafana_api_key:
            self.headers['Authorization'] = "Bearer %s" % self.grafana_api_key
        else:
            self.force_basic_auth = True

        if self.grafana_url is None:
            self.disabled = True
            self._display.warning('Grafana URL was not provided. The '
                                  'Grafana URL can be provided using '
                                  'the `GRAFANA_URL` environment variable.')
        self._display.debug('Grafana URL: %s' % self.grafana_url)

    def v2_playbook_on_start(self, playbook):
        self.playbook = playbook._file_name
        text = PLAYBOOK_START_TXT.format(playbook=self.playbook, hostname=self.hostname,
                                         username=self.username)
        data = {
            'time': to_millis(self.start_time),
            'text': text,
            'tags': ['ansible', 'ansible_event_start', self.playbook, self.hostname]
        }
        self._send_annotation(data)

    def v2_playbook_on_stats(self, stats):
        end_time = datetime.now()
        duration = end_time - self.start_time
        summarize_stat = {}
        for host in stats.processed.keys():
            summarize_stat[host] = stats.summarize(host)

        status = "FAILED"
        if self.errors == 0:
            status = "OK"

        text = PLAYBOOK_STATS_TXT.format(playbook=self.playbook, hostname=self.hostname,
                                         duration=duration.total_seconds(),
                                         status=status, username=self.username,
                                         summary=json.dumps(summarize_stat))

        data = {
            'time': to_millis(self.start_time),
            'timeEnd': to_millis(end_time),
            'isRegion': True,
            'text': text,
            'tags': ['ansible', 'ansible_report', self.playbook, self.hostname]
        }
        self._send_annotations(data)

    def v2_runner_on_failed(self, result, ignore_errors=False, **kwargs):
        text = PLAYBOOK_ERROR_TXT.format(playbook=self.playbook, hostname=self.hostname,
                                         username=self.username, task=result._task,
                                         host=result._host.name, result=self._dump_results(result._result))
        if ignore_errors:
            return
        data = {
            'time': to_millis(datetime.now()),
            'text': text,
            'tags': ['ansible', 'ansible_event_failure', self.playbook, self.hostname]
        }
        self.errors += 1
        self._send_annotations(data)

    def _send_annotations(self, data):
        if self.dashboard_id:
            data["dashboardId"] = int(self.dashboard_id)
        if self.panel_ids:
            for panel_id in self.panel_ids:
                data["panelId"] = int(panel_id)
                self._send_annotation(data)
        else:
            self._send_annotation(data)

    def _send_annotation(self, annotation):
        try:
            open_url(self.grafana_url, data=json.dumps(annotation), headers=self.headers,
                     method="POST",
                     validate_certs=self.validate_grafana_certs,
                     url_username=self.grafana_user, url_password=self.grafana_password,
                     http_agent=self.http_agent, force_basic_auth=self.force_basic_auth)
        except Exception as e:
            self._display.error(u'Could not submit message to Grafana: %s' % to_text(e))



#!/usr/bin/python

# Copyright: (c) 2018, Rhys Campbell <rhys.james.campbell@googlemail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: mongodb_status
short_description: Validates the status of the cluster.
description:
  - Validates the status of the cluster.
  - The module expects all replicaset nodes to be PRIMARY, SECONDARY or ARBITER.
  - Will wait until a timeout for the replicaset state to converge if required.
  - Can also be used to lookup the current PRIMARY member (see examples).
author: Rhys Campbell (@rhysmeister)
version_added: "1.0.0"

extends_documentation_fragment:
  - community.mongodb.login_options
  - community.mongodb.ssl_options

options:
  replica_set:
    description:
    - Replicaset name.
    type: str
    default: rs0
  poll:
    description:
      - The maximum number of times to query for the replicaset status before the set converges or we fail.
    type: int
    default: 1
  interval:
    description:
      - The number of seconds to wait between polling executions.
    type: int
    default: 30
notes:
- Requires the pymongo Python package on the remote host, version 2.4.2+. This
  can be installed using pip or the OS package manager. @see U(http://api.mongodb.org/python/current/installation.html)
requirements:
- pymongo
'''

EXAMPLES = r'''
- name: Check replicaset is healthy, fail if not after first attempt
  community.mongodb.mongodb_status:
    replica_set: rs0
  when: ansible_hostname == "mongodb1"

- name: Wait for the replicaset rs0 to converge, check 5 times, 10 second interval between checks
  community.mongodb.mongodb_status:
    replica_set: rs0
    poll: 5
    interval: 10
  when: ansible_hostname == "mongodb1"

# Get the replicaset status and then lookup the primary's hostname and save to a variable
- name: Ensure replicaset is stable before beginning
  community.mongodb.mongodb_status:
    login_user: "{{ admin_user }}"
    login_password: "{{ admin_user_password }}"
    poll: 3
    interval: 10
  register: rs

- name: Lookup PRIMARY replicaset member
  set_fact:
    primary: "{{ item.key.split('.')[0] }}"
  loop: "{{ lookup('dict', rs.replicaset) }}"
  when: "'PRIMARY' in item.value"
'''

RETURN = r'''
failed:
  description: If the mnodule had failed or not.
  returned: always
  type: bool
iterations:
  description: Number of times the module has queried the replicaset status.
  returned: always
  type: int
msg:
  description: Status message.
  returned: always
  type: str
replicaset:
  description: The last queried status of all the members of the replicaset if obtainable.
  returned: always
  type: dict
'''


from copy import deepcopy
import time

import os
import ssl as ssl_lib
from distutils.version import LooseVersion
import traceback


from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.six import binary_type, text_type
from ansible.module_utils.six.moves import configparser
from ansible.module_utils._text import to_native
from ansible_collections.community.mongodb.plugins.module_utils.mongodb_common import (
    check_compatibility,
    missing_required_lib,
    load_mongocnf,
    mongodb_common_argument_spec,
    ssl_connection_options
)
from ansible_collections.community.mongodb.plugins.module_utils.mongodb_common import PyMongoVersion, PYMONGO_IMP_ERR, pymongo_found, MongoClient


def replicaset_status(client, module):
    """
    Return the replicaset status document from MongoDB
    # https://docs.mongodb.com/manual/reference/command/replSetGetStatus/
    """
    rs = client.admin.command('replSetGetStatus')
    return rs


def replicaset_members(replicaset_document):
    """
    Returns the members section of the MongoDB replicaset document
    """
    return replicaset_document["members"]


def replicaset_friendly_document(members_document):
    """
    Returns a version of the members document with
    only the info this module requires: name & stateStr
    """
    friendly_document = {}

    for member in members_document:
        friendly_document[member["name"]] = member["stateStr"]
    return friendly_document


def replicaset_statuses(members_document, module):
    """
    Return a list of the statuses
    """
    statuses = []
    for member in members_document:
        statuses.append(members_document[member])
    return statuses


def replicaset_good(statuses, module):
    """
    Returns true if the replicaset is in a "good" condition.
    Good is defined as an odd number of servers >= 3, with
    max one primary, and any even amount of
    secondary and arbiter servers
    """
    module.debug(msg=str(statuses))
    msg = "Unset"
    status = None
    valid_statuses = ["PRIMARY", "SECONDARY", "ARBITER"]
    # Odd number of servers is good
    if len(statuses) % 2 == 1:
        if (statuses.count("PRIMARY") == 1
            and ((statuses.count("SECONDARY")
                  + statuses.count("ARBITER")) % 2 == 0)
                and len(set(statuses) - set(valid_statuses)) == 0):
            status = True
            msg = "replicaset is in a converged state"
        else:
            status = False
            msg = "replicaset is not currently in a converged state"
    else:
        msg = "Even number of servers currently in replicaset."
        status = False
    return status, msg


def replicaset_status_poll(client, module):
    """
    client - MongoDB Client
    poll - Number of times to poll
    interval - interval between polling attempts
    """
    iterations = 0  # How many times we have queried the cluster
    failures = 0  # Number of failures when querying the replicaset
    poll = module.params['poll']
    interval = module.params['interval']
    status = None
    return_doc = {}

    while iterations < poll:
        try:
            iterations += 1
            replicaset_document = replicaset_status(client, module)
            members = replicaset_members(replicaset_document)
            friendly_document = replicaset_friendly_document(members)
            statuses = replicaset_statuses(friendly_document, module)
            status, msg = replicaset_good(statuses, module)
            if status:  # replicaset looks good
                return_doc = {"failures": failures,
                              "poll": poll,
                              "iterations": iterations,
                              "msg": msg,
                              "replicaset": friendly_document}
                break
            else:
                failures += 1
                return_doc = {"failures": failures,
                              "poll": poll,
                              "iterations": iterations,
                              "msg": msg,
                              "replicaset": friendly_document,
                              "failed": True}
                if iterations == poll:
                    break
                else:
                    time.sleep(interval)
        except Exception as e:
            failures += 1
            return_doc['failed'] = True
            return_doc['msg'] = str(e)
            status = False
            if iterations == poll:
                break
            else:
                time.sleep(interval)

    return_doc['failures'] = failures
    return status, return_doc['msg'], return_doc


# =========================================
# Module execution.
#


def main():
    argument_spec = mongodb_common_argument_spec()
    argument_spec.update(
        interval=dict(type='int', default=30),
        poll=dict(type='int', default=1),
        replica_set=dict(type='str', default="rs0"),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=False,
        required_together=[['login_user', 'login_password']],
    )
    if not pymongo_found:
        module.fail_json(msg=missing_required_lib('pymongo'),
                         exception=PYMONGO_IMP_ERR)

    login_user = module.params['login_user']
    login_password = module.params['login_password']
    login_database = module.params['login_database']
    login_host = module.params['login_host']
    login_port = module.params['login_port']
    replica_set = module.params['replica_set']
    ssl = module.params['ssl']
    poll = module.params['poll']
    interval = module.params['interval']

    result = dict(
        failed=False,
        replica_set=replica_set,
    )

    connection_params = dict(
        host=login_host,
        port=int(login_port),
    )

    if ssl:
        connection_params = ssl_connection_options(connection_params, module)

    try:
        client = MongoClient(**connection_params)
    except Exception as e:
        module.fail_json(msg='Unable to connect to database: %s' % to_native(e))

    try:
        # Get server version:
        try:
            srv_version = LooseVersion(client.server_info()['version'])
        except Exception as e:
            module.fail_json(msg='Unable to get MongoDB server version: %s' % to_native(e))

        # Get driver version::
        driver_version = LooseVersion(PyMongoVersion)

        # Check driver and server version compatibility:
        check_compatibility(module, srv_version, driver_version)
    except Exception as excep:
        if excep.code != 13:
            raise excep
        if login_user is None or login_password is None:
            raise excep
        client.admin.authenticate(login_user, login_password, source=login_database)
        check_compatibility(module, client)

    if login_user is None and login_password is None:
        mongocnf_creds = load_mongocnf()
        if mongocnf_creds is not False:
            login_user = mongocnf_creds['user']
            login_password = mongocnf_creds['password']
    elif login_password is None or login_user is None:
        module.fail_json(msg="When supplying login arguments, both 'login_user' and 'login_password' must be provided")

    try:
        client['admin'].command('listDatabases', 1.0)  # if this throws an error we need to authenticate
    except Exception as excep:
        if "not authorized on" in str(excep) or "command listDatabases requires authentication" in str(excep):
            if login_user is not None and login_password is not None:
                try:
                    client.admin.authenticate(login_user, login_password, source=login_database)
                except Exception as excep:
                    module.fail_json(msg='unable to connect to database: %s' % to_native(excep), exception=traceback.format_exc())
            else:
                module.fail_json(msg='unable to connect to database: %s' % to_native(excep), exception=traceback.format_exc())
        else:
            module.fail_json(msg='unable to connect to database: %s' % to_native(excep), exception=traceback.format_exc())

    if len(replica_set) == 0:
        module.fail_json(msg="Parameter 'replica_set' must not be an empty string")

    try:
        status, msg, return_doc = replicaset_status_poll(client, module)  # Sort out the return doc
        replicaset = return_doc['replicaset']
        iterations = return_doc['iterations']
    except Exception as e:
        module.fail_json(msg='Unable to query replica_set info: %s' % str(e))

    if status is False:
        module.fail_json(msg=msg, replicaset=replicaset, iterations=iterations)
    else:
        module.exit_json(msg=msg, replicaset=replicaset, iterations=iterations)


if __name__ == '__main__':
    main()



# Copyright: (c) 2020, quidame <quidame@poivron.org>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

import time

from ansible.plugins.action import ActionBase
from ansible.errors import AnsibleError, AnsibleActionFail, AnsibleConnectionFailure
from ansible.utils.vars import merge_hash
from ansible.utils.display import Display

display = Display()


class ActionModule(ActionBase):

    # Keep internal params away from user interactions
    _VALID_ARGS = frozenset(('path', 'state', 'table', 'noflush', 'counters', 'modprobe', 'ip_version', 'wait'))
    DEFAULT_SUDOABLE = True

    MSG_ERROR__ASYNC_AND_POLL_NOT_ZERO = (
        "This module doesn't support async>0 and poll>0 when its 'state' param "
        "is set to 'restored'. To enable its rollback feature (that needs the "
        "module to run asynchronously on the remote), please set task attribute "
        "'poll' (=%s) to 0, and 'async' (=%s) to a value >2 and not greater than "
        "'ansible_timeout' (=%s) (recommended).")
    MSG_WARNING__NO_ASYNC_IS_NO_ROLLBACK = (
        "Attempts to restore iptables state without rollback in case of mistake "
        "may lead the ansible controller to loose access to the hosts and never "
        "regain it before fixing firewall rules through a serial console, or any "
        "other way except SSH. Please set task attribute 'poll' (=%s) to 0, and "
        "'async' (=%s) to a value >2 and not greater than 'ansible_timeout' (=%s) "
        "(recommended).")
    MSG_WARNING__ASYNC_GREATER_THAN_TIMEOUT = (
        "You attempt to restore iptables state with rollback in case of mistake, "
        "but with settings that will lead this rollback to happen AFTER that the "
        "controller will reach its own timeout. Please set task attribute 'poll' "
        "(=%s) to 0, and 'async' (=%s) to a value >2 and not greater than "
        "'ansible_timeout' (=%s) (recommended).")

    def _async_result(self, module_args, task_vars, timeout):
        '''
        Retrieve results of the asynchonous task, and display them in place of
        the async wrapper results (those with the ansible_job_id key).
        '''
        # At least one iteration is required, even if timeout is 0.
        for i in range(max(1, timeout)):
            async_result = self._execute_module(
                module_name='ansible.builtin.async_status',
                module_args=module_args,
                task_vars=task_vars,
                wrap_async=False)
            if async_result['finished'] == 1:
                break
            time.sleep(min(1, timeout))

        return async_result

    def run(self, tmp=None, task_vars=None):

        self._supports_check_mode = True
        self._supports_async = True

        result = super(ActionModule, self).run(tmp, task_vars)
        del tmp  # tmp no longer has any effect

        if not result.get('skipped'):

            # FUTURE: better to let _execute_module calculate this internally?
            wrap_async = self._task.async_val and not self._connection.has_native_async

            # Set short names for values we'll have to compare or reuse
            task_poll = self._task.poll
            task_async = self._task.async_val
            check_mode = self._play_context.check_mode
            max_timeout = self._connection._play_context.timeout
            module_name = self._task.action
            module_args = self._task.args

            if module_args.get('state', None) == 'restored':
                if not wrap_async:
                    if not check_mode:
                        display.warning(self.MSG_WARNING__NO_ASYNC_IS_NO_ROLLBACK % (
                            task_poll,
                            task_async,
                            max_timeout))
                elif task_poll:
                    raise AnsibleActionFail(self.MSG_ERROR__ASYNC_AND_POLL_NOT_ZERO % (
                        task_poll,
                        task_async,
                        max_timeout))
                else:
                    if task_async > max_timeout and not check_mode:
                        display.warning(self.MSG_WARNING__ASYNC_GREATER_THAN_TIMEOUT % (
                            task_poll,
                            task_async,
                            max_timeout))

                    # BEGIN snippet from async_status action plugin
                    env_async_dir = [e for e in self._task.environment if
                                     "ANSIBLE_ASYNC_DIR" in e]
                    if len(env_async_dir) > 0:
                        # for backwards compatibility we need to get the dir from
                        # ANSIBLE_ASYNC_DIR that is defined in the environment. This is
                        # deprecated and will be removed in favour of shell options
                        async_dir = env_async_dir[0]['ANSIBLE_ASYNC_DIR']

                        msg = "Setting the async dir from the environment keyword " \
                              "ANSIBLE_ASYNC_DIR is deprecated. Set the async_dir " \
                              "shell option instead"
                        display.deprecated(msg, version='2.0.0',
                                           collection_name='community.general')  # was Ansible 2.12
                    else:
                        # inject the async directory based on the shell option into the
                        # module args
                        async_dir = self.get_shell_option('async_dir', default="~/.ansible_async")
                    # END snippet from async_status action plugin

                    # Bind the loop max duration to consistent values on both
                    # remote and local sides (if not the same, make the loop
                    # longer on the controller); and set a backup file path.
                    module_args['_timeout'] = task_async
                    module_args['_back'] = '%s/iptables.state' % async_dir
                    async_status_args = dict(_async_dir=async_dir)
                    confirm_cmd = 'rm -f %s' % module_args['_back']
                    starter_cmd = 'touch %s.starter' % module_args['_back']
                    remaining_time = max(task_async, max_timeout)

            # do work!
            result = merge_hash(result, self._execute_module(module_args=module_args, task_vars=task_vars, wrap_async=wrap_async))

            # Then the 3-steps "go ahead or rollback":
            # 1. Catch early errors of the module (in asynchronous task) if any.
            #    Touch a file on the target to signal the module to process now.
            # 2. Reset connection to ensure a persistent one will not be reused.
            # 3. Confirm the restored state by removing the backup on the remote.
            #    Retrieve the results of the asynchronous task to return them.
            if '_back' in module_args:
                async_status_args['jid'] = result.get('ansible_job_id', None)
                if async_status_args['jid'] is None:
                    raise AnsibleActionFail("Unable to get 'ansible_job_id'.")

                # Catch early errors due to missing mandatory option, bad
                # option type/value, missing required system command, etc.
                result = merge_hash(result, self._async_result(async_status_args, task_vars, 0))

                # The module is aware to not process the main iptables-restore
                # command before finding (and deleting) the 'starter' cookie on
                # the host, so the previous query will not reach ssh timeout.
                garbage = self._low_level_execute_command(starter_cmd, sudoable=self.DEFAULT_SUDOABLE)

                # As the main command is not yet executed on the target, here
                # 'finished' means 'failed before main command be executed'.
                if not result['finished']:
                    try:
                        self._connection.reset()
                    except AttributeError:
                        pass

                    for x in range(max_timeout):
                        time.sleep(1)
                        remaining_time -= 1
                        # - AnsibleConnectionFailure covers rejected requests (i.e.
                        #   by rules with '--jump REJECT')
                        # - ansible_timeout is able to cover dropped requests (due
                        #   to a rule or policy DROP) if not lower than async_val.
                        try:
                            garbage = self._low_level_execute_command(confirm_cmd, sudoable=self.DEFAULT_SUDOABLE)
                            break
                        except AnsibleConnectionFailure:
                            continue

                    result = merge_hash(result, self._async_result(async_status_args, task_vars, remaining_time))

                # Cleanup async related stuff and internal params
                for key in ('ansible_job_id', 'results_file', 'started', 'finished'):
                    if result.get(key):
                        del result[key]

                if result.get('invocation', {}).get('module_args'):
                    if '_timeout' in result['invocation']['module_args']:
                        del result['invocation']['module_args']['_back']
                        del result['invocation']['module_args']['_timeout']

                async_status_args['mode'] = 'cleanup'
                garbage = self._execute_module(
                    module_name='ansible.builtin.async_status',
                    module_args=async_status_args,
                    task_vars=task_vars,
                    wrap_async=False)

        if not wrap_async:
            # remove a temporary path we created
            self._remove_tmp_path(self._connection._shell.tmpdir)

        return result



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


DOCUMENTATION = r'''
---
module: ce_lldp_interface
version_added: '0.2.0'
short_description: Manages INTERFACE LLDP configuration on HUAWEI CloudEngine switches.
description:
    - Manages INTERFACE LLDP configuration on HUAWEI CloudEngine switches.
author: xuxiaowei0512 (@CloudEngine-Ansible)
notes:
    - This module requires the netconf system service be enabled on the remote device being managed.
    - Recommended connection is C(netconf).
    - This module also works with C(local) connections for legacy playbooks.
options:
  lldpenable:
    description:
      - Set global LLDP enable state.
    type: str
    choices: ['enabled', 'disabled']
  function_lldp_interface_flag:
    description:
      - Used to distinguish between command line functions.
    type: str
    choices: ['disableINTERFACE','tlvdisableINTERFACE','tlvenableINTERFACE','intervalINTERFACE']
  type_tlv_disable:
    description:
      - Used to distinguish between command line functions.
    type: str
    choices: ['basic_tlv', 'dot3_tlv']
  type_tlv_enable:
    description:
      - Used to distinguish between command line functions.
    type: str
    choices: ['dot1_tlv','dcbx']
  lldpadminstatus:
    description:
      - Set interface lldp enable state.
    type: str
    choices: ['txOnly', 'rxOnly', 'txAndRx', 'disabled']
  ifname:
    description:
      - Interface name.
    type: str
  txinterval:
    description:
      - LLDP send message interval.
    type: int
  txprotocolvlanid:
    description:
      - Set tx protocol vlan id.
    type: int
  txvlannameid:
    description:
      - Set tx vlan name id.
    type: int
  vlannametxenable:
    description:
      - Set vlan name tx enable or not.
    type: bool
  manaddrtxenable:
    description:
      - Make it able to send management address TLV.
    type: bool
  portdesctxenable:
    description:
      - Enabling the ability to send a description of TLV.
    type: bool
  syscaptxenable:
    description:
      - Enable the ability to send system capabilities TLV.
    type: bool
  sysdesctxenable:
    description:
      - Enable the ability to send system description TLV.
    type: bool
  sysnametxenable:
    description:
      - Enable the ability to send system name TLV.
    type: bool
  portvlantxenable:
    description:
      - Enable port vlan tx.
    type: bool
  protovlantxenable:
    description:
      - Enable protocol vlan tx.
    type: bool
  protoidtxenable:
     description:
       - Enable the ability to send protocol identity TLV.
     type: bool
  macphytxenable:
    description:
      - Enable MAC/PHY configuration and state TLV to be sent.
    type: bool
  linkaggretxenable:
    description:
      - Enable the ability to send link aggregation TLV.
    type: bool
  maxframetxenable:
    description:
      - Enable the ability to send maximum frame length TLV.
    type: bool
  eee:
    description:
      - Enable the ability to send EEE TLV.
    type: bool
  dcbx:
    description:
      - Enable the ability to send DCBX TLV.
    type: bool
  state:
    description:
      - Manage the state of the resource.
    type: str
    default: present
    choices: ['present', 'absent']
'''

EXAMPLES = '''
  - name: "Configure global LLDP enable state"
    ce_lldp_interface_interface:
      lldpenable: enabled

  - name: "Configure interface lldp enable state"
    community.network.ce_lldp_interface:
      function_lldp_interface_flag: disableINTERFACE
      ifname: 10GE1/0/1
      lldpadminstatus: rxOnly
  - name: "Configure LLDP transmit interval and ensure global LLDP state is already enabled"
    community.network.ce_lldp_interface:
      function_lldp_interface_flag: intervalINTERFACE
      ifname: 10GE1/0/1
      txinterval: 4

  - name: "Configure basic-tlv: management-address TLV"
    community.network.ce_lldp_interface:
      function_lldp_interface_flag: tlvdisableINTERFACE
      type_tlv_disable: basic_tlv
      ifname: 10GE1/0/1
      manaddrtxenable: true

  - name: "Configure basic-tlv: prot description TLV"
    community.network.ce_lldp_interface:
      function_lldp_interface_flag: tlvdisableINTERFACE
      type_tlv_disable: basic_tlv
      ifname: 10GE1/0/1
      portdesctxenable: true

  - name: "Configure basic-tlv: system capabilities TLV"
    community.network.ce_lldp_interface:
      function_lldp_interface_flag: tlvdisableINTERFACE
      type_tlv_disable: basic_tlv
      ifname: 10GE1/0/1
      syscaptxenable: true

  - name: "Configure basic-tlv: system description TLV"
    community.network.ce_lldp_interface:
      function_lldp_interface_flag: tlvdisableINTERFACE
      type_tlv_disable: basic_tlv
      ifname: 10GE1/0/1
      sysdesctxenable: true

  - name: "Configure basic-tlv: system name TLV"
    community.network.ce_lldp_interface:
      function_lldp_interface_flag: tlvdisableINTERFACE
      type_tlv_disable: basic_tlv
      ifname: 10GE1/0/1
      sysnametxenable: true

  - name: "TLV types that are forbidden to be published on the configuration interface, link aggregation TLV"
    community.network.ce_lldp_interface:
      function_lldp_interface_flag: tlvdisableINTERFACE
      type_tlv_disable: dot3_tlv
      ifname: 10GE1/0/1
      linkAggreTxEnable: true

  - name: "TLV types that are forbidden to be published on the configuration interface, MAC/PHY configuration/status TLV"
    community.network.ce_lldp_interface:
      function_lldp_interface_flag: tlvdisableINTERFACE
      type_tlv_disable: dot3_tlv
      ifname: 10GE1/0/1
      macPhyTxEnable: true

  - name: "TLV types that are forbidden to be published on the configuration interface, maximum frame size TLV"
    community.network.ce_lldp_interface:
      function_lldp_interface_flag: tlvdisableINTERFACE
      type_tlv_disable: dot3_tlv
      ifname: 10GE1/0/1
      maxFrameTxEnable: true

  - name: "TLV types that are forbidden to be published on the configuration interface, EEE TLV"
    community.network.ce_lldp_interface:
      function_lldp_interface_flag: tlvdisableINTERFACE
      type_tlv_disable: dot3_tlv
      ifname: 10GE1/0/1
      eee: true

  - name: "Configure the interface to publish an optional DCBX TLV type "
    community.network.ce_lldp_interface:
      function_lldp_interface_flag: tlvenableINTERFACE
      ifname: 10GE1/0/1
      type_tlv_enable: dcbx
'''

RETURN = '''
proposed:
  description: k/v pairs of parameters passed into module
  returned: always
  type: dict
  sample: {
          "lldpenable": "enabled",
          "lldpadminstatus": "rxOnly",
          "function_lldp_interface_flag": "tlvenableINTERFACE",
          "type_tlv_enable": "dot1_tlv",
          "ifname": "10GE1/0/1",
          "state": "present"
          }
existing:
  description: k/v pairs of existing global LLDP configration
  returned: always
  type: dict
  sample: {
           "lldpenable": "disabled",
           "ifname": "10GE1/0/1",
           "lldpadminstatus": "txAndRx"
          }
end_state:
  description: k/v pairs of global DLDP configration after module execution
  returned: always
  type: dict
  sample: {
           "lldpenable": "enabled",
           "lldpadminstatus": "rxOnly",
           "function_lldp_interface_flag": "tlvenableINTERFACE",
           "type_tlv_enable": "dot1_tlv",
           "ifname": "10GE1/0/1"
          }
updates:
  description: command sent to the device
  returned: always
  type: list
  sample: [
           "lldp enable",
           "interface 10ge 1/0/1",
           "undo lldp disable",
           "lldp tlv-enable dot1-tlv vlan-name 4",
           ]
changed:
  description: check to see if a change was made on the device
  returned: always
  type: bool
  sample: true
'''

import copy
import re
from xml.etree import ElementTree
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.network.plugins.module_utils.network.cloudengine.ce import set_nc_config, get_nc_config

CE_NC_GET_GLOBAL_LLDPENABLE_CONFIG = """
<filter type="subtree">
  <lldp xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
    <lldpSys>
      <lldpEnable></lldpEnable>
    </lldpSys>
  </lldp>
</filter>
"""

CE_NC_MERGE_GLOBA_LLDPENABLE_CONFIG = """
<config>
  <lldp xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
    <lldpSys operation="merge">
      <lldpEnable>%s</lldpEnable>
    </lldpSys>
  </lldp>
</config>
"""

CE_NC_GET_INTERFACE_LLDP_CONFIG = """
<filter type="subtree">
  <lldp xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
    <lldpInterfaces>
      <lldpInterface>
        <ifName></ifName>
        <lldpAdminStatus></lldpAdminStatus>
      </lldpInterface>
    </lldpInterfaces>
  </lldp>
</filter>
"""

CE_NC_MERGE_INTERFACE_LLDP_CONFIG = """
<config>
  <lldp xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
    <lldpInterfaces>
      <lldpInterface operation="merge">
        <ifName>%s</ifName>
        <lldpAdminStatus>%s</lldpAdminStatus>
      </lldpInterface>
    </lldpInterfaces>
  </lldp>
</config>
"""

CE_NC_GET_INTERFACE_INTERVAl_CONFIG = """
<filter type="subtree">
  <lldp xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
    <lldpInterfaces>
      <lldpInterface>
        <ifName></ifName>
        <msgInterval>
          <txInterval></txInterval>
        </msgInterval>
      </lldpInterface>
    </lldpInterfaces>
  </lldp>
</filter>
"""

CE_NC_MERGE_INTERFACE_INTERVAl_CONFIG = """
<config>
  <lldp xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
    <lldpInterfaces>
      <lldpInterface>
        <ifName>%s</ifName>
        <msgInterval operation="merge">
          <txInterval>%s</txInterval>
        </msgInterval>
      </lldpInterface>
    </lldpInterfaces>
  </lldp>
</config>
"""

CE_NC_GET_INTERFACE_TLV_ENABLE_CONFIG = """
<filter type="subtree">
    <lldp xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <lldpInterfaces>
            <lldpInterface>
                <ifName></ifName>
                <tlvTxEnable>
                    <dcbx></dcbx>
                    <protoIdTxEnable></protoIdTxEnable>
                </tlvTxEnable>
            </lldpInterface>
        </lldpInterfaces>
    </lldp>
</filter>
"""

CE_NC_GET_INTERFACE_TLV_DISABLE_CONFIG = """
<filter type="subtree">
    <lldp xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <lldpInterfaces>
            <lldpInterface>
                <ifName></ifName>
                <tlvTxEnable>
                    <manAddrTxEnable></manAddrTxEnable>
                    <portDescTxEnable></portDescTxEnable>
                    <sysCapTxEnable></sysCapTxEnable>
                    <sysDescTxEnable></sysDescTxEnable>
                    <sysNameTxEnable></sysNameTxEnable>
                    <linkAggreTxEnable></linkAggreTxEnable>
                    <macPhyTxEnable></macPhyTxEnable>
                    <maxFrameTxEnable></maxFrameTxEnable>
                    <eee></eee>
                </tlvTxEnable>
            </lldpInterface>
        </lldpInterfaces>
    </lldp>
</filter>
"""

CE_NC_MERGE_INTERFACE_TLV_CONFIG_HEADER = """
<config>
    <lldp xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <lldpInterfaces>
            <lldpInterface>
                <ifName>%s</ifName>
                <tlvTxEnable operation="merge">
"""

CE_NC_MERGE_INTERFACE_TLV_CONFIG_ENABLE_PROTOIDTXENABLE = """
                    <protoIdTxEnable>%s</protoIdTxEnable>
"""

CE_NC_MERGE_INTERFACE_TLV_CONFIG_ENABLE_DCBX = """
                    <dcbx>%s</dcbx>
"""

CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_MANADDRTXENABLE = """
                    <manAddrTxEnable>%s</manAddrTxEnable>
"""

CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_PORTDESCTXENABLE = """
                    <portDescTxEnable>%s</portDescTxEnable>
"""

CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_SYSCAPTXENABLE = """
                    <sysCapTxEnable>%s</sysCapTxEnable>
"""

CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_SYSDESCTXENABLE = """
                    <sysDescTxEnable>%s</sysDescTxEnable>
"""

CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_SYSNAMETXENABLE = """
                    <sysNameTxEnable>%s</sysNameTxEnable>
"""

CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_LINKAGGRETXENABLE = """
                    <linkAggreTxEnable>%s</linkAggreTxEnable>
"""

CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_MACPHYTXENABLE = """
                    <macPhyTxEnable>%s</macPhyTxEnable>
"""

CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_MAXFRAMETXENABLE = """
                    <maxFrameTxEnable>%s</maxFrameTxEnable>
"""

CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_EEE = """
                    <eee>%s</eee>
"""

CE_NC_MERGE_INTERFACE_TLV_CONFIG_TAIL = """
                </tlvTxEnable>
            </lldpInterface>
        </lldpInterfaces>
    </lldp>
</config>
"""

CE_NC_MERGE_GLOBA_LLDPENABLE_CONFIG = """
<config>
    <lldp xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <lldpSys operation="merge">
            <lldpEnable>%s</lldpEnable>
        </lldpSys>
    </lldp>
</config>
"""


def get_interface_type(interface):
    """Gets the type of interface, such as 10GE"""

    if interface is None:
        return None

    iftype = None

    if interface.upper().startswith('GE'):
        iftype = 'ge'
    elif interface.upper().startswith('10GE'):
        iftype = '10ge'
    elif interface.upper().startswith('25GE'):
        iftype = '25ge'
    elif interface.upper().startswith('40GE'):
        iftype = '40ge'
    elif interface.upper().startswith('100GE'):
        iftype = '100ge'
    elif interface.upper().startswith('PORT-GROUP'):
        iftype = 'stack-Port'
    elif interface.upper().startswith('NULL'):
        iftype = 'null'
    else:
        return None
    return iftype.lower()


class Lldp_interface(object):
    """Manage global lldp enable configuration"""

    def __init__(self, argument_spec):
        self.spec = argument_spec
        self.module = AnsibleModule(argument_spec=self.spec, supports_check_mode=True)

        self.lldpenable = self.module.params['lldpenable'] or None
        self.function_lldp_interface_flag = self.module.params['function_lldp_interface_flag']
        self.type_tlv_disable = self.module.params['type_tlv_disable']
        self.type_tlv_enable = self.module.params['type_tlv_enable']
        self.ifname = self.module.params['ifname']
        if self.function_lldp_interface_flag == 'disableINTERFACE':
            self.ifname = self.module.params['ifname']
            self.lldpadminstatus = self.module.params['lldpadminstatus']
        elif self.function_lldp_interface_flag == 'tlvdisableINTERFACE':
            if self.type_tlv_disable == 'basic_tlv':
                self.ifname = self.module.params['ifname']
                self.manaddrtxenable = self.module.params['manaddrtxenable']
                self.portdesctxenable = self.module.params['portdesctxenable']
                self.syscaptxenable = self.module.params['syscaptxenable']
                self.sysdesctxenable = self.module.params['sysdesctxenable']
                self.sysnametxenable = self.module.params['sysnametxenable']
            if self.type_tlv_disable == 'dot3_tlv':
                self.ifname = self.module.params['ifname']
                self.macphytxenable = self.module.params['macphytxenable']
                self.linkaggretxenable = self.module.params['linkaggretxenable']
                self.maxframetxenable = self.module.params['maxframetxenable']
                self.eee = self.module.params['eee']
        elif self.function_lldp_interface_flag == 'tlvenableINTERFACE':
            if self.type_tlv_enable == 'dot1_tlv':
                self.ifname = self.module.params['ifname']
                self.protoidtxenable = self.module.params['protoidtxenable']
            if self.type_tlv_enable == 'dcbx':
                self.ifname = self.module.params['ifname']
                self.dcbx = self.module.params['dcbx']
        elif self.function_lldp_interface_flag == 'intervalINTERFACE':
            self.ifname = self.module.params['ifname']
            self.txinterval = self.module.params['txinterval']
        self.state = self.module.params['state']

        self.lldp_conf = dict()
        self.conf_disable_exsit = False
        self.conf_interface_lldp_disable_exsit = False
        self.conf_interval_exsit = False
        self.conf_tlv_disable_exsit = False
        self.conf_tlv_enable_exsit = False
        self.enable_flag = 0
        self.check_params()
        self.existing_state_value = dict()
        self.existing_end_state_value = dict()
        self.interface_lldp_info = list()

        # state
        self.changed = False
        self.proposed_changed = dict()
        self.updates_cmd = list()
        self.results = dict()
        self.proposed = dict()
        self.existing = dict()
        self.end_state = dict()

    def check_params(self):
        """Check all input params"""

        if self.ifname:
            intf_type = get_interface_type(self.ifname)
            if not intf_type:
                self.module.fail_json(msg='Error: ifname name of %s is error.' % self.ifname)
            if (len(self.ifname) < 1) or (len(self.ifname) > 63):
                self.module.fail_json(msg='Error: Ifname length is beetween 1 and 63.')

        if self.function_lldp_interface_flag == 'intervalINTERFACE':
            if self.txinterval:
                if int(self.txinterval) < 1 or int(self.txinterval) > 32768:
                    self.module.fail_json(
                        msg='Error: The value of txinterval is out of [1 - 32768].')
                if self.ifname:
                    intf_type = get_interface_type(self.ifname)
                    if not intf_type:
                        self.module.fail_json(
                            msg='Error: ifname name of %s '
                                'is error.' % self.ifname)
                    if (len(self.ifname) < 1) or (len(self.ifname) > 63):
                        self.module.fail_json(
                            msg='Error: Ifname length is beetween 1 and 63.')

        if self.function_lldp_interface_flag == 'tlvdisableINTERFACE':
            if self.type_tlv_disable == 'dot1_tlv':
                if self.ifname:
                    intf_type = get_interface_type(self.ifname)
                    if not intf_type:
                        self.module.fail_json(
                            msg='Error: ifname name of %s '
                                'is error.' % self.ifname)
                    if (len(self.ifname) < 1) or (len(self.ifname) > 63):
                        self.module.fail_json(
                            msg='Error: Ifname length is beetween 1 and 63.')

        if self.function_lldp_interface_flag == 'tlvenableINTERFACE':
            if self.type_tlv_enable == 'dot1_tlv':
                if self.ifname:
                    intf_type = get_interface_type(self.ifname)
                    if not intf_type:
                        self.module.fail_json(
                            msg='Error: ifname name of %s '
                                'is error.' % self.ifname)
                    if (len(self.ifname) < 1) or (len(self.ifname) > 63):
                        self.module.fail_json(
                            msg='Error: Ifname length is beetween 1 and 63.')

    def check_response(self, xml_str, xml_name):
        """Check if response message is already OK"""

        if "<ok/>" not in xml_str:
            self.module.fail_json(msg='Error: %s failed.' % xml_name)

    def show_result(self):
        """Show result"""

        self.results['changed'] = self.changed
        self.results['proposed'] = self.proposed
        self.results['existing'] = self.existing
        self.results['end_state'] = self.end_state
        if self.changed:
            self.results['updates'] = self.updates_cmd
        else:
            self.results['updates'] = list()

        self.module.exit_json(**self.results)

    def get_lldp_enable_pre_config(self):
        """Get lldp enable configure"""

        lldp_dict = dict()
        lldp_config = list()
        conf_enable_str = CE_NC_GET_GLOBAL_LLDPENABLE_CONFIG
        conf_enable_obj = get_nc_config(self.module, conf_enable_str)
        xml_enable_str = conf_enable_obj.replace('\r', '').replace('\n', '').\
            replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "").\
            replace('xmlns="http://www.huawei.com/netconf/vrp"', "")

        # get lldp enable config info
        root_enable = ElementTree.fromstring(xml_enable_str)
        ntpsite_enable = root_enable.findall("lldp/lldpSys")
        for nexthop_enable in ntpsite_enable:
            for ele_enable in nexthop_enable:
                if ele_enable.tag in ["lldpEnable"]:
                    lldp_dict[ele_enable.tag] = ele_enable.text
                    if lldp_dict['lldpEnable'] == 'enabled':
                        self.enable_flag = 1
            lldp_config.append(dict(lldpenable=lldp_dict['lldpEnable']))
        return lldp_config

    def get_interface_lldp_disable_pre_config(self):
        """Get interface undo lldp disable configure"""
        lldp_dict = dict()
        interface_lldp_disable_dict = dict()
        if self.enable_flag == 1:
            conf_enable_str = CE_NC_GET_INTERFACE_LLDP_CONFIG
            conf_enable_obj = get_nc_config(self.module, conf_enable_str)
            if "<data/>" in conf_enable_obj:
                return
            xml_enable_str = conf_enable_obj.replace('\r', '').replace('\n', '').\
                replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "").\
                replace('xmlns="http://www.huawei.com/netconf/vrp"', "")
            root = ElementTree.fromstring(xml_enable_str)
            lldp_disable_enable = root.findall("lldp/lldpInterfaces/lldpInterface")
            for nexthop_enable in lldp_disable_enable:
                name = nexthop_enable.find("ifName")
                status = nexthop_enable.find("lldpAdminStatus")
                if name is not None and status is not None:
                    interface_lldp_disable_dict[name.text] = status.text
        return interface_lldp_disable_dict

    def get_interface_lldp_disable_config(self):
        lldp_config = list()
        interface_lldp_disable_dict_tmp = dict()
        if self.state == "present":
            if self.ifname:
                interface_lldp_disable_dict_tmp = self.get_interface_lldp_disable_pre_config()
                key_list = interface_lldp_disable_dict_tmp.keys()
                if len(key_list) != 0:
                    for key in key_list:
                        if key == self.ifname:
                            if interface_lldp_disable_dict_tmp[key] != self.lldpadminstatus:
                                self.conf_interface_lldp_disable_exsit = True
                            else:
                                self.conf_interface_lldp_disable_exsit = False
                elif self.ifname not in key_list:
                    self.conf_interface_lldp_disable_exsit = True
                elif (len(key_list) == 0) and self.ifname and self.lldpadminstatus:
                    self.conf_interface_lldp_disable_exsit = True
                lldp_config.append(interface_lldp_disable_dict_tmp)
        return lldp_config

    def get_interface_tlv_disable_config(self):
        lldp_config = list()
        lldp_dict = dict()
        cur_interface_mdn_cfg = dict()
        exp_interface_mdn_cfg = dict()

        if self.enable_flag == 1:
            conf_str = CE_NC_GET_INTERFACE_TLV_DISABLE_CONFIG
            conf_obj = get_nc_config(self.module, conf_str)
            if "<data/>" in conf_obj:
                return lldp_config
            xml_str = conf_obj.replace('\r', '').replace('\n', '')
            xml_str = xml_str.replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "")
            xml_str = xml_str.replace('xmlns="http://www.huawei.com/netconf/vrp"', "")
            root = ElementTree.fromstring(xml_str)
            lldp_tlvdisable_ifname = root.findall("lldp/lldpInterfaces/lldpInterface")
            for ele in lldp_tlvdisable_ifname:
                ifname_tmp = ele.find("ifName")
                manaddrtxenable_tmp = ele.find("tlvTxEnable/manAddrTxEnable")
                portdesctxenable_tmp = ele.find("tlvTxEnable/portDescTxEnable")
                syscaptxenable_tmp = ele.find("tlvTxEnable/sysCapTxEnable")
                sysdesctxenable_tmp = ele.find("tlvTxEnable/sysDescTxEnable")
                sysnametxenable_tmp = ele.find("tlvTxEnable/sysNameTxEnable")
                linkaggretxenable_tmp = ele.find("tlvTxEnable/linkAggreTxEnable")
                macphytxenable_tmp = ele.find("tlvTxEnable/macPhyTxEnable")
                maxframetxenable_tmp = ele.find("tlvTxEnable/maxFrameTxEnable")
                eee_tmp = ele.find("tlvTxEnable/eee")
                if ifname_tmp is not None:
                    if ifname_tmp.text is not None:
                        cur_interface_mdn_cfg["ifname"] = ifname_tmp.text
                if ifname_tmp is not None and manaddrtxenable_tmp is not None:
                    if manaddrtxenable_tmp.text is not None:
                        cur_interface_mdn_cfg["manaddrtxenable"] = manaddrtxenable_tmp.text
                if ifname_tmp is not None and portdesctxenable_tmp is not None:
                    if portdesctxenable_tmp.text is not None:
                        cur_interface_mdn_cfg['portdesctxenable'] = portdesctxenable_tmp.text
                if ifname_tmp is not None and syscaptxenable_tmp is not None:
                    if syscaptxenable_tmp.text is not None:
                        cur_interface_mdn_cfg['syscaptxenable'] = syscaptxenable_tmp.text
                if ifname_tmp is not None and sysdesctxenable_tmp is not None:
                    if sysdesctxenable_tmp.text is not None:
                        cur_interface_mdn_cfg['sysdesctxenable'] = sysdesctxenable_tmp.text
                if ifname_tmp is not None and sysnametxenable_tmp is not None:
                    if sysnametxenable_tmp.text is not None:
                        cur_interface_mdn_cfg['sysnametxenable'] = sysnametxenable_tmp.text
                if ifname_tmp is not None and linkaggretxenable_tmp is not None:
                    if linkaggretxenable_tmp.text is not None:
                        cur_interface_mdn_cfg['linkaggretxenable'] = linkaggretxenable_tmp.text
                if ifname_tmp is not None and macphytxenable_tmp is not None:
                    if macphytxenable_tmp.text is not None:
                        cur_interface_mdn_cfg['macphytxenable'] = macphytxenable_tmp.text
                if ifname_tmp is not None and maxframetxenable_tmp is not None:
                    if maxframetxenable_tmp.text is not None:
                        cur_interface_mdn_cfg['maxframetxenable'] = maxframetxenable_tmp.text
                if ifname_tmp is not None and eee_tmp is not None:
                    if eee_tmp.text is not None:
                        cur_interface_mdn_cfg['eee'] = eee_tmp.text
                if self.state == "present":
                    if self.function_lldp_interface_flag == 'tlvdisableINTERFACE':
                        if self.type_tlv_disable == 'basic_tlv':
                            if self.ifname:
                                exp_interface_mdn_cfg['ifname'] = self.ifname
                                if self.manaddrtxenable:
                                    exp_interface_mdn_cfg['manaddrtxenable'] = self.manaddrtxenable
                                if self.portdesctxenable:
                                    exp_interface_mdn_cfg['portdesctxenable'] = self.portdesctxenable
                                if self.syscaptxenable:
                                    exp_interface_mdn_cfg['syscaptxenable'] = self.syscaptxenable
                                if self.sysdesctxenable:
                                    exp_interface_mdn_cfg['sysdesctxenable'] = self.sysdesctxenable
                                if self.sysnametxenable:
                                    exp_interface_mdn_cfg['sysnametxenable'] = self.sysnametxenable
                                if self.ifname == ifname_tmp.text:
                                    key_list = exp_interface_mdn_cfg.keys()
                                    key_list_cur = cur_interface_mdn_cfg.keys()
                                    if len(key_list) != 0:
                                        for key in key_list:
                                            if key == "ifname" and self.ifname == cur_interface_mdn_cfg['ifname']:
                                                lldp_config.append(dict(ifname=cur_interface_mdn_cfg['ifname']))
                                            if "manaddrtxenable" == key and self.ifname == cur_interface_mdn_cfg['ifname']:
                                                lldp_config.append(dict(manaddrtxenable=cur_interface_mdn_cfg['manaddrtxenable']))
                                            if "portdesctxenable" == key and self.ifname == cur_interface_mdn_cfg['ifname']:
                                                lldp_config.append(dict(portdesctxenable=cur_interface_mdn_cfg['portdesctxenable']))
                                            if "syscaptxenable" == key and self.ifname == cur_interface_mdn_cfg['ifname']:
                                                lldp_config.append(dict(syscaptxenable=cur_interface_mdn_cfg['syscaptxenable']))
                                            if "sysdesctxenable" == key and self.ifname == cur_interface_mdn_cfg['ifname']:
                                                lldp_config.append(dict(sysdesctxenable=cur_interface_mdn_cfg['sysdesctxenable']))
                                            if "sysnametxenable" == key and self.ifname == cur_interface_mdn_cfg['ifname']:
                                                lldp_config.append(dict(sysnametxenable=cur_interface_mdn_cfg['sysnametxenable']))
                                            if key in key_list_cur:
                                                if str(exp_interface_mdn_cfg[key]) != str(cur_interface_mdn_cfg[key]):
                                                    self.conf_tlv_disable_exsit = True
                                                    self.changed = True
                                                    return lldp_config
                                            else:
                                                self.conf_tlv_disable_exsit = True
                                                return lldp_config

                        if self.type_tlv_disable == 'dot3_tlv':
                            if self.ifname:
                                exp_interface_mdn_cfg['ifname'] = self.ifname
                                if self.linkaggretxenable:
                                    exp_interface_mdn_cfg['linkaggretxenable'] = self.linkaggretxenable
                                if self.macphytxenable:
                                    exp_interface_mdn_cfg['macphytxenable'] = self.macphytxenable
                                if self.maxframetxenable:
                                    exp_interface_mdn_cfg['maxframetxenable'] = self.maxframetxenable
                                if self.eee:
                                    exp_interface_mdn_cfg['eee'] = self.eee
                                if self.ifname == ifname_tmp.text:
                                    key_list = exp_interface_mdn_cfg.keys()
                                    key_list_cur = cur_interface_mdn_cfg.keys()
                                    if len(key_list) != 0:
                                        for key in key_list:
                                            if key == "ifname" and self.ifname == cur_interface_mdn_cfg['ifname']:
                                                lldp_config.append(dict(ifname=cur_interface_mdn_cfg['ifname']))
                                            if "linkaggretxenable" == key and self.ifname == cur_interface_mdn_cfg['ifname']:
                                                lldp_config.append(dict(linkaggretxenable=cur_interface_mdn_cfg['linkaggretxenable']))
                                            if "macphytxenable" == key and self.ifname == cur_interface_mdn_cfg['ifname']:
                                                lldp_config.append(dict(macphytxenable=cur_interface_mdn_cfg['macphytxenable']))
                                            if "maxframetxenable" == key and self.ifname == cur_interface_mdn_cfg['ifname']:
                                                lldp_config.append(dict(maxframetxenable=cur_interface_mdn_cfg['maxframetxenable']))
                                            if "eee" == key and self.ifname == cur_interface_mdn_cfg['ifname']:
                                                lldp_config.append(dict(eee=cur_interface_mdn_cfg['eee']))
                                            if key in key_list_cur:
                                                if str(exp_interface_mdn_cfg[key]) != str(cur_interface_mdn_cfg[key]):
                                                    self.conf_tlv_disable_exsit = True
                                                    self.changed = True
                                                    return lldp_config
                                            else:
                                                self.conf_tlv_disable_exsit = True
                                                return lldp_config
        return lldp_config

    def get_interface_tlv_enable_config(self):
        lldp_config = list()
        lldp_dict = dict()
        cur_interface_mdn_cfg = dict()
        exp_interface_mdn_cfg = dict()
        if self.enable_flag == 1:
            conf_str = CE_NC_GET_INTERFACE_TLV_ENABLE_CONFIG
            conf_obj = get_nc_config(self.module, conf_str)
            if "<data/>" in conf_obj:
                return lldp_config
            xml_str = conf_obj.replace('\r', '').replace('\n', '')
            xml_str = xml_str.replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "")
            xml_str = xml_str.replace('xmlns="http://www.huawei.com/netconf/vrp"', "")
            root = ElementTree.fromstring(xml_str)
            lldpenablesite = root.findall("lldp/lldpInterfaces/lldpInterface")
            for ele in lldpenablesite:
                ifname_tmp = ele.find("ifName")
                protoidtxenable_tmp = ele.find("tlvTxEnable/protoIdTxEnable")
                dcbx_tmp = ele.find("tlvTxEnable/dcbx")
                if ifname_tmp is not None:
                    if ifname_tmp.text is not None:
                        cur_interface_mdn_cfg["ifname"] = ifname_tmp.text
                if ifname_tmp is not None and protoidtxenable_tmp is not None:
                    if protoidtxenable_tmp.text is not None:
                        cur_interface_mdn_cfg["protoidtxenable"] = protoidtxenable_tmp.text
                if ifname_tmp is not None and dcbx_tmp is not None:
                    if dcbx_tmp.text is not None:
                        cur_interface_mdn_cfg['dcbx'] = dcbx_tmp.text
                if self.state == "present":
                    if self.function_lldp_interface_flag == 'tlvenableINTERFACE':
                        if self.type_tlv_enable == 'dot1_tlv':
                            if self.ifname:
                                exp_interface_mdn_cfg['ifname'] = self.ifname
                                if self.protoidtxenable:
                                    exp_interface_mdn_cfg['protoidtxenable'] = self.protoidtxenable
                                if self.ifname == ifname_tmp.text:
                                    key_list = exp_interface_mdn_cfg.keys()
                                    key_list_cur = cur_interface_mdn_cfg.keys()
                                    if len(key_list) != 0:
                                        for key in key_list:
                                            if "protoidtxenable" == str(key) and self.ifname == cur_interface_mdn_cfg['ifname']:
                                                lldp_config.append(dict(protoidtxenable=cur_interface_mdn_cfg['protoidtxenable']))
                                            if key in key_list_cur:
                                                if str(exp_interface_mdn_cfg[key]) != str(cur_interface_mdn_cfg[key]):
                                                    self.conf_tlv_enable_exsit = True
                                                    self.changed = True
                                                    return lldp_config
                                            else:
                                                self.conf_tlv_enable_exsit = True
                                                return lldp_config
                        if self.type_tlv_enable == 'dcbx':
                            if self.ifname:
                                exp_interface_mdn_cfg['ifname'] = self.ifname
                                if self.dcbx:
                                    exp_interface_mdn_cfg['dcbx'] = self.dcbx
                                if self.ifname == ifname_tmp.text:
                                    key_list = exp_interface_mdn_cfg.keys()
                                    key_list_cur = cur_interface_mdn_cfg.keys()
                                    if len(key_list) != 0:
                                        for key in key_list:
                                            if "dcbx" == key and self.ifname == cur_interface_mdn_cfg['ifname']:
                                                lldp_config.append(dict(dcbx=cur_interface_mdn_cfg['dcbx']))
                                            if key in key_list_cur:
                                                if str(exp_interface_mdn_cfg[key]) != str(cur_interface_mdn_cfg[key]):
                                                    self.conf_tlv_enable_exsit = True
                                                    self.changed = True
                                                    return lldp_config
                                            else:
                                                self.conf_tlv_enable_exsit = True
                                                return lldp_config
        return lldp_config

    def get_interface_interval_config(self):
        lldp_config = list()
        lldp_dict = dict()
        cur_interface_mdn_cfg = dict()
        exp_interface_mdn_cfg = dict()
        interface_lldp_disable_dict_tmp2 = self.get_interface_lldp_disable_pre_config()
        if self.enable_flag == 1:
            if interface_lldp_disable_dict_tmp2[self.ifname] != 'disabled':
                conf_str = CE_NC_GET_INTERFACE_INTERVAl_CONFIG
                conf_obj = get_nc_config(self.module, conf_str)
                if "<data/>" in conf_obj:
                    return lldp_config
                xml_str = conf_obj.replace('\r', '').replace('\n', '')
                xml_str = xml_str.replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "")
                xml_str = xml_str.replace('xmlns="http://www.huawei.com/netconf/vrp"', "")
                root = ElementTree.fromstring(xml_str)
                txintervalsite = root.findall("lldp/lldpInterfaces/lldpInterface")
                for ele in txintervalsite:
                    ifname_tmp = ele.find("ifName")
                    txinterval_tmp = ele.find("msgInterval/txInterval")
                    if ifname_tmp is not None:
                        if ifname_tmp.text is not None:
                            cur_interface_mdn_cfg["ifname"] = ifname_tmp.text
                    if txinterval_tmp is not None:
                        if txinterval_tmp.text is not None:
                            cur_interface_mdn_cfg["txinterval"] = txinterval_tmp.text
                    if self.state == "present":
                        if self.ifname:
                            exp_interface_mdn_cfg["ifname"] = self.ifname
                            if self.txinterval:
                                exp_interface_mdn_cfg["txinterval"] = self.txinterval
                            if self.ifname == ifname_tmp.text:
                                key_list = exp_interface_mdn_cfg.keys()
                                key_list_cur = cur_interface_mdn_cfg.keys()
                                if len(key_list) != 0:
                                    for key in key_list:
                                        if "txinterval" == str(key) and self.ifname == cur_interface_mdn_cfg['ifname']:
                                            lldp_config.append(dict(ifname=cur_interface_mdn_cfg['ifname'], txinterval=exp_interface_mdn_cfg['txinterval']))
                                        if key in key_list_cur:
                                            if str(exp_interface_mdn_cfg[key]) != str(cur_interface_mdn_cfg[key]):
                                                self.conf_interval_exsit = True
                                                lldp_config.append(cur_interface_mdn_cfg)
                                                return lldp_config
                                        else:
                                            self.conf_interval_exsit = True
                                            return lldp_config
        return lldp_config

    def config_global_lldp_enable(self):
        if self.state == 'present':
            if self.enable_flag == 0 and self.lldpenable == 'enabled':
                xml_str = CE_NC_MERGE_GLOBA_LLDPENABLE_CONFIG % self.lldpenable
                ret_xml = set_nc_config(self.module, xml_str)
                self.check_response(ret_xml, "LLDP_ENABLE_CONFIG")
                self.changed = True
            elif self.enable_flag == 1 and self.lldpenable == 'disabled':
                xml_str = CE_NC_MERGE_GLOBA_LLDPENABLE_CONFIG % self.lldpenable
                ret_xml = set_nc_config(self.module, xml_str)
                self.check_response(ret_xml, "LLDP_ENABLE_CONFIG")
                self.changed = True

    def config_interface_lldp_disable_config(self):
        if self.function_lldp_interface_flag == 'disableINTERFACE':
            if self.enable_flag == 1 and self.conf_interface_lldp_disable_exsit:
                if self.ifname:
                    xml_str = CE_NC_MERGE_INTERFACE_LLDP_CONFIG % (self.ifname, self.lldpadminstatus)
                    ret_xml = set_nc_config(self.module, xml_str)
                    self.check_response(ret_xml, "INTERFACE_LLDP_DISABLE_CONFIG")
                    self.changed = True

    def config_interface_tlv_disable_config(self):
        if self.function_lldp_interface_flag == 'tlvdisableINTERFACE':
            if self.enable_flag == 1 and self.conf_tlv_disable_exsit:
                if self.type_tlv_disable == 'basic_tlv':
                    if self.ifname:
                        if self.portdesctxenable:
                            xml_str = (CE_NC_MERGE_INTERFACE_TLV_CONFIG_HEADER % self.ifname) + \
                                (CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_PORTDESCTXENABLE % self.portdesctxenable) + \
                                CE_NC_MERGE_INTERFACE_TLV_CONFIG_TAIL
                            ret_xml = set_nc_config(self.module, xml_str)
                            self.check_response(ret_xml, "TLV_DISABLE_PORTDESCTXENABLE")
                            self.changed = True
                        if self.manaddrtxenable:
                            xml_str = (CE_NC_MERGE_INTERFACE_TLV_CONFIG_HEADER % self.ifname) + \
                                (CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_MANADDRTXENABLE % self.manaddrtxenable) + \
                                CE_NC_MERGE_INTERFACE_TLV_CONFIG_TAIL
                            ret_xml = set_nc_config(self.module, xml_str)
                            self.check_response(ret_xml, "TLV_DISABLE_MANADDRTXENABLE")
                            self.changed = True
                        if self.syscaptxenable:
                            xml_str = (CE_NC_MERGE_INTERFACE_TLV_CONFIG_HEADER % self.ifname) + \
                                (CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_SYSCAPTXENABLE % self.syscaptxenable) + \
                                CE_NC_MERGE_INTERFACE_TLV_CONFIG_TAIL
                            ret_xml = set_nc_config(self.module, xml_str)
                            self.check_response(ret_xml, "TLV_DISABLE_SYSCAPTXENABLE")
                            self.changed = True
                        if self.sysdesctxenable:
                            xml_str = (CE_NC_MERGE_INTERFACE_TLV_CONFIG_HEADER % self.ifname) + \
                                (CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_SYSDESCTXENABLE % self.sysdesctxenable) + \
                                CE_NC_MERGE_INTERFACE_TLV_CONFIG_TAIL
                            ret_xml = set_nc_config(self.module, xml_str)
                            self.check_response(ret_xml, "TLV_DISABLE_SYSDESCTXENABLE")
                            self.changed = True
                        if self.sysnametxenable:
                            xml_str = (CE_NC_MERGE_INTERFACE_TLV_CONFIG_HEADER % self.ifname) + \
                                (CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_SYSNAMETXENABLE % self.sysnametxenable) + \
                                CE_NC_MERGE_INTERFACE_TLV_CONFIG_TAIL
                            ret_xml = set_nc_config(self.module, xml_str)
                            self.check_response(ret_xml, "TLV_DISABLE_SYSNAMETXENABLE")
                            self.changed = True
                if self.type_tlv_disable == 'dot3_tlv':
                    if self.ifname:
                        if self.linkaggretxenable:
                            xml_str = (CE_NC_MERGE_INTERFACE_TLV_CONFIG_HEADER % self.ifname) + \
                                (CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_LINKAGGRETXENABLE % self.linkaggretxenable) + \
                                CE_NC_MERGE_INTERFACE_TLV_CONFIG_TAIL
                            ret_xml = set_nc_config(self.module, xml_str)
                            self.check_response(ret_xml, "TLV_DISABLE_LINKAGGRETXENABLE")
                            self.changed = True
                        if self.macphytxenable:
                            xml_str = (CE_NC_MERGE_INTERFACE_TLV_CONFIG_HEADER % self.ifname) + \
                                (CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_MACPHYTXENABLE % self.macphytxenable) + \
                                CE_NC_MERGE_INTERFACE_TLV_CONFIG_TAIL
                            ret_xml = set_nc_config(self.module, xml_str)
                            self.check_response(ret_xml, "TLV_DISABLE_MACPHYTXENABLE")
                            self.changed = True
                        if self.maxframetxenable:
                            xml_str = (CE_NC_MERGE_INTERFACE_TLV_CONFIG_HEADER % self.ifname) + \
                                (CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_MAXFRAMETXENABLE % self.maxframetxenable) + \
                                CE_NC_MERGE_INTERFACE_TLV_CONFIG_TAIL
                            ret_xml = set_nc_config(self.module, xml_str)
                            self.check_response(ret_xml, "TLV_DISABLE_MAXFRAMETXENABLE")
                            self.changed = True
                        if self.eee:
                            xml_str = (CE_NC_MERGE_INTERFACE_TLV_CONFIG_HEADER % self.ifname) + \
                                (CE_NC_MERGE_INTERFACE_TLV_CONFIG_DISABLE_EEE % self.eee) + \
                                CE_NC_MERGE_INTERFACE_TLV_CONFIG_TAIL
                            ret_xml = set_nc_config(self.module, xml_str)
                            self.check_response(ret_xml, "TLV_DISABLE_EEE")
                            self.changed = True

    def config_interface_tlv_enable_config(self):
        if self.function_lldp_interface_flag == 'tlvenableINTERFACE':
            if self.enable_flag == 1 and self.conf_tlv_enable_exsit:
                if self.type_tlv_enable == 'dot1_tlv':
                    if self.ifname:
                        if self.protoidtxenable:
                            xml_str = (CE_NC_MERGE_INTERFACE_TLV_CONFIG_HEADER % self.ifname) + \
                                (CE_NC_MERGE_INTERFACE_TLV_CONFIG_ENABLE_PROTOIDTXENABLE % self.protoidtxenable) + \
                                CE_NC_MERGE_INTERFACE_TLV_CONFIG_TAIL
                            ret_xml = set_nc_config(self.module, xml_str)
                            self.check_response(ret_xml, "TLV_ENABLE_DOT1_PORT_VLAN")
                            self.changed = True
                if self.type_tlv_enable == 'dcbx':
                    if self.ifname:
                        if self.dcbx:
                            xml_str = (CE_NC_MERGE_INTERFACE_TLV_CONFIG_HEADER % self.ifname) + \
                                (CE_NC_MERGE_INTERFACE_TLV_CONFIG_ENABLE_DCBX % self.dcbx) + \
                                CE_NC_MERGE_INTERFACE_TLV_CONFIG_TAIL
                            ret_xml = set_nc_config(self.module, xml_str)
                            self.check_response(ret_xml, "TLV_ENABLE_DCBX_VLAN")
                            self.changed = True

    def config_interface_interval_config(self):
        if self.function_lldp_interface_flag == 'intervalINTERFACE':
            tmp = self.get_interface_lldp_disable_pre_config()
            if self.enable_flag == 1 and self.conf_interval_exsit and tmp[self.ifname] != 'disabled':
                if self.ifname:
                    if self.txinterval:
                        xml_str = CE_NC_MERGE_INTERFACE_INTERVAl_CONFIG % (self.ifname, self.txinterval)
                        ret_xml = set_nc_config(self.module, xml_str)
                        self.check_response(ret_xml, "INTERFACE_INTERVAL_CONFIG")
                        self.changed = True

    def get_existing(self):
        """get existing information"""
        self.get_lldp_enable_pre_config()
        if self.lldpenable:
            self.existing['globalLLDPENABLE'] = self.get_lldp_enable_pre_config()
        if self.function_lldp_interface_flag == 'disableINTERFACE':
            self.existing['disableINTERFACE'] = self.get_interface_lldp_disable_config()
        if self.function_lldp_interface_flag == 'tlvdisableINTERFACE':
            self.existing['tlvdisableINTERFACE'] = self.get_interface_tlv_disable_config()
        if self.function_lldp_interface_flag == 'tlvenableINTERFACE':
            self.existing['tlvenableINTERFACE'] = self.get_interface_tlv_enable_config()
        if self.function_lldp_interface_flag == 'intervalINTERFACE':
            self.existing['intervalINTERFACE'] = self.get_interface_interval_config()

    def get_proposed(self):
        """get proposed"""
        if self.lldpenable:
            self.proposed = dict(lldpenable=self.lldpenable)
        if self.function_lldp_interface_flag == 'disableINTERFACE':
            if self.enable_flag == 1:
                self.proposed = dict(ifname=self.ifname, lldpadminstatus=self.lldpadminstatus)
        if self.function_lldp_interface_flag == 'tlvdisableINTERFACE':
            if self.enable_flag == 1:
                if self.type_tlv_disable == 'basic_tlv':
                    if self.ifname:
                        if self.manaddrtxenable:
                            self.proposed = dict(ifname=self.ifname, manaddrtxenable=self.manaddrtxenable)
                        if self.portdesctxenable:
                            self.proposed = dict(ifname=self.ifname, portdesctxenable=self.portdesctxenable)
                        if self.syscaptxenable:
                            self.proposed = dict(ifname=self.ifname, syscaptxenable=self.syscaptxenable)
                        if self.sysdesctxenable:
                            self.proposed = dict(ifname=self.ifname, sysdesctxenable=self.sysdesctxenable)
                        if self.sysnametxenable:
                            self.proposed = dict(ifname=self.ifname, sysnametxenable=self.sysnametxenable)
                if self.type_tlv_disable == 'dot3_tlv':
                    if self.ifname:
                        if self.linkaggretxenable:
                            self.proposed = dict(ifname=self.ifname, linkaggretxenable=self.linkaggretxenable)
                        if self.macphytxenable:
                            self.proposed = dict(ifname=self.ifname, macphytxenable=self.macphytxenable)
                        if self.maxframetxenable:
                            self.proposed = dict(ifname=self.ifname, maxframetxenable=self.maxframetxenable)
                        if self.eee:
                            self.proposed = dict(ifname=self.ifname, eee=self.eee)
        if self.function_lldp_interface_flag == 'tlvenableINTERFACE':
            if self.enable_flag == 1:
                if self.type_tlv_enable == 'dot1_tlv':
                    if self.ifname:
                        if self.protoidtxenable:
                            self.proposed = dict(ifname=self.ifname, protoidtxenable=self.protoidtxenable)
                if self.type_tlv_enable == 'dcbx':
                    if self.ifname:
                        if self.dcbx:
                            self.proposed = dict(ifname=self.ifname, dcbx=self.dcbx)
        if self.function_lldp_interface_flag == 'intervalINTERFACE':
            tmp1 = self.get_interface_lldp_disable_pre_config()
            if self.enable_flag == 1 and tmp1[self.ifname] != 'disabled':
                self.proposed = dict(ifname=self.ifname, txinterval=self.txinterval)

    def config_lldp_interface(self):
        """config lldp interface"""
        if self.lldpenable:
            self.config_global_lldp_enable()
        if self.function_lldp_interface_flag == 'disableINTERFACE':
            self.config_interface_lldp_disable_config()
        elif self.function_lldp_interface_flag == 'tlvdisableINTERFACE':
            self.config_interface_tlv_disable_config()
        elif self.function_lldp_interface_flag == 'tlvenableINTERFACE':
            self.config_interface_tlv_enable_config()
        elif self.function_lldp_interface_flag == 'intervalINTERFACE':
            self.config_interface_interval_config()

    def get_end_state(self):
        """get end_state information"""
        self.get_lldp_enable_pre_config()
        if self.lldpenable:
            self.end_state['globalLLDPENABLE'] = self.get_lldp_enable_pre_config()
        if self.function_lldp_interface_flag == 'disableINTERFACE':
            self.end_state['disableINTERFACE'] = self.get_interface_lldp_disable_config()
        if self.function_lldp_interface_flag == 'tlvdisableINTERFACE':
            self.end_state['tlvdisableINTERFACE'] = self.get_interface_tlv_disable_config()
        if self.function_lldp_interface_flag == 'tlvenableINTERFACE':
            self.end_state['tlvenableINTERFACE'] = self.get_interface_tlv_enable_config()
        if self.function_lldp_interface_flag == 'intervalINTERFACE':
            self.end_state['intervalINTERFACE'] = self.get_interface_interval_config()

    def get_update_cmd(self):
        """Get updated commands"""

        cmds = []
        if self.state == "present":
            if self.lldpenable == "enabled":
                cmds.append("lldp enable")
                if self.function_lldp_interface_flag == 'disableINTERFACE':
                    if self.ifname:
                        cmds.append("%s %s" % ("interface", self.ifname))
                        if self.lldpadminstatus == 'disabled':
                            cmds.append("lldp disable")
                        else:
                            cmds.append("undo lldp disable")
                elif self.function_lldp_interface_flag == 'tlvdisableINTERFACE':
                    if self.type_tlv_disable == 'basic_tlv':
                        if self.ifname:
                            cmds.append("%s %s" % ("interface", self.ifname))
                            if self.manaddrtxenable:
                                if self.manaddrtxenable == "false":
                                    cmds.append("lldp tlv-disable basic-tlv management-address")
                                if self.manaddrtxenable == "true":
                                    cmds.append("undo lldp tlv-disable basic-tlv management-address")
                            if self.portdesctxenable:
                                if self.portdesctxenable == "false":
                                    cmds.append("lldp tlv-disable basic-tlv port-description")
                                if self.portdesctxenable == "true":
                                    cmds.append("undo lldp tlv-disable basic-tlv port-description")
                            if self.syscaptxenable:
                                if self.syscaptxenable == "false":
                                    cmds.append("lldp tlv-disable basic-tlv system-capability")
                                if self.syscaptxenable == "true":
                                    cmds.append("undo lldp tlv-disable basic-tlv system-capability")
                            if self.sysdesctxenable:
                                if self.sysdesctxenable == "false":
                                    cmds.append("lldp tlv-disable basic-tlv system-description")
                                if self.sysdesctxenable == "true":
                                    cmds.append("undo lldp tlv-disable basic-tlv system-description")
                            if self.sysnametxenable:
                                if self.sysnametxenable == "false":
                                    cmds.append("lldp tlv-disable basic-tlv system-name")
                                if self.sysnametxenable == "true":
                                    cmds.append("undo lldp tlv-disable basic-tlv system-name")
                    if self.type_tlv_disable == 'dot3_tlv':
                        if self.ifname:
                            cmds.append("%s %s" % ("interface", self.ifname))
                            if self.linkaggretxenable:
                                if self.linkaggretxenable == "false":
                                    cmds.append("lldp tlv-disable dot3-tlv link-aggregation")
                                if self.linkaggretxenable == "true":
                                    cmds.append("undo lldp tlv-disable dot3-tlv link-aggregation")
                            if self.macphytxenable:
                                if self.macphytxenable == "false":
                                    cmds.append("lldp tlv-disable dot3-tlv mac-physic")
                                if self.macphytxenable == "true":
                                    cmds.append("undo lldp tlv-disable dot3-tlv mac-physic")
                            if self.maxframetxenable:
                                if self.maxframetxenable == "false":
                                    cmds.append("lldp tlv-disable dot3-tlv max-frame-size")
                                if self.maxframetxenable == "true":
                                    cmds.append("undo lldp tlv-disable dot3-tlv max-frame-size")
                            if self.eee:
                                if self.eee == "false":
                                    cmds.append("lldp tlv-disable dot3-tlv eee")
                                if self.eee == "true":
                                    cmds.append("undo lldp tlv-disable dot3-tlv eee")
                elif self.function_lldp_interface_flag == 'tlvenableINTERFACE':
                    if self.type_tlv_enable == 'dot1_tlv':
                        if self.ifname:
                            cmds.append("%s %s" % ("interface", self.ifname))
                            if self.protoidtxenable:
                                if self.protoidtxenable == "false":
                                    cmds.append("undo lldp tlv-enable dot1-tlv protocol-identity")
                                if self.protoidtxenable == "true":
                                    cmds.append("lldp tlv-enable dot1-tlv protocol-identity")
                    if self.type_tlv_enable == 'dcbx':
                        if self.ifname:
                            cmds.append("%s %s" % ("interface", self.ifname))
                            if self.dcbx:
                                if self.dcbx == "false":
                                    cmds.append("undo lldp tlv-enable dcbx")
                                if self.dcbx == "true":
                                    cmds.append("lldp tlv-enable dcbx")
                elif self.function_lldp_interface_flag == 'intervalINTERFACE':
                    if self.ifname:
                        cmds.append("%s %s" % ("interface", self.ifname))
                        if self.txinterval:
                            cmds.append("lldp transmit fast-mode interval %s" % self.txinterval)
            elif self.lldpenable == "disabled":
                cmds.append("undo lldp enable")
            else:
                if self.enable_flag == 1:
                    if self.function_lldp_interface_flag == 'disableINTERFACE':
                        if self.ifname:
                            cmds.append("interface %s" % self.ifname)
                            if self.lldpadminstatus == 'disabled':
                                cmds.append("lldp disable")
                            else:
                                cmds.append("undo lldp disable")
                    elif self.function_lldp_interface_flag == 'tlvdisableINTERFACE':
                        if self.type_tlv_disable == 'basic_tlv':
                            if self.ifname:
                                cmds.append("interface %s" % self.ifname)
                                if self.manaddrtxenable:
                                    if self.manaddrtxenable == "false":
                                        cmds.append("lldp tlv-disable basic-tlv management-address")
                                    if self.manaddrtxenable == "true":
                                        cmds.append("undo lldp tlv-disable basic-tlv management-address")
                                if self.portdesctxenable:
                                    if self.portdesctxenable == "false":
                                        cmds.append("lldp tlv-disable basic-tlv port-description")
                                    if self.portdesctxenable == "true":
                                        cmds.append("undo lldp tlv-disable basic-tlv port-description")
                                if self.syscaptxenable:
                                    if self.syscaptxenable == "false":
                                        cmds.append("lldp tlv-disable basic-tlv system-capability")
                                    if self.syscaptxenable == "true":
                                        cmds.append("undo lldp tlv-disable basic-tlv system-capability")
                                if self.sysdesctxenable:
                                    if self.sysdesctxenable == "false":
                                        cmds.append("lldp tlv-disable basic-tlv system-description")
                                    if self.sysdesctxenable == "true":
                                        cli_str = "%s %s\n" % (cli_str, "undo lldp tlv-disable basic-tlv system-description")
                                if self.sysnametxenable:
                                    if self.sysnametxenable == "false":
                                        cmds.append("lldp tlv-disable basic-tlv system-name")
                                    if self.sysnametxenable == "true":
                                        cmds.append("undo lldp tlv-disable basic-tlv system-name")
                        if self.type_tlv_disable == 'dot3_tlv':
                            if self.ifname:
                                cmds.append("interface %s" % self.ifname)
                                if self.linkaggretxenable:
                                    if self.linkaggretxenable == "false":
                                        cmds.append("lldp tlv-disable dot3-tlv link-aggregation")
                                    if self.linkaggretxenable == "true":
                                        cmds.append("undo lldp tlv-disable dot3-tlv link-aggregation")
                                if self.macphytxenable:
                                    if self.macphytxenable == "false":
                                        cmds.append("lldp tlv-disable dot3-tlv mac-physic")
                                    if self.macphytxenable == "true":
                                        cli_str = "%s %s\n" % (cli_str, "undo lldp tlv-disable dot3-tlv mac-physic")
                                if self.maxframetxenable:
                                    if self.maxframetxenable == "false":
                                        cmds.append("lldp tlv-disable dot3-tlv max-frame-size")
                                    if self.maxframetxenable == "true":
                                        cmds.append("undo lldp tlv-disable dot3-tlv max-frame-size")
                                if self.eee:
                                    if self.eee == "false":
                                        cmds.append("lldp tlv-disable dot3-tlv eee")
                                    if self.eee == "true":
                                        cmds.append("undo lldp tlv-disable dot3-tlv eee")
                    elif self.function_lldp_interface_flag == 'tlvenableINTERFACE':
                        if self.type_tlv_enable == 'dot1_tlv':
                            if self.ifname:
                                cmds.append("interface %s" % self.ifname)
                                if self.protoidtxenable:
                                    if self.protoidtxenable == "false":
                                        cmds.append("undo lldp tlv-enable dot1-tlv protocol-identity")
                                    if self.protoidtxenable == "true":
                                        cmds.append("lldp tlv-enable dot1-tlv protocol-identity")
                        if self.type_tlv_enable == 'dcbx':
                            if self.ifname:
                                cmds.append("interface %s" % self.ifname)
                                if self.dcbx:
                                    if self.dcbx == "false":
                                        cmds.append("undo lldp tlv-enable dcbx")
                                    if self.dcbx == "true":
                                        cmds.append("lldp tlv-enable dcbx")
                    elif self.function_lldp_interface_flag == 'intervalINTERFACE':
                        if self.ifname:
                            cmds.append("interface %s" % self.ifname)
                            if self.txinterval:
                                cmds.append("lldp transmit fast-mode interval %s" % self.txinterval)
        self.updates_cmd = cmds

    def work(self):
        """Execute task"""
        self.check_params()
        self.get_existing()
        self.get_proposed()
        self.config_lldp_interface()
        self.get_update_cmd()
        self.get_end_state()
        self.show_result()


def main():
    """Main function"""

    argument_spec = dict(
        lldpenable=dict(choices=['enabled', 'disabled']),
        function_lldp_interface_flag=dict(choices=['disableINTERFACE', 'tlvdisableINTERFACE', 'tlvenableINTERFACE', 'intervalINTERFACE'], type='str'),
        type_tlv_disable=dict(choices=['basic_tlv', 'dot3_tlv'], type='str'),
        type_tlv_enable=dict(choices=['dot1_tlv', 'dcbx'], type='str'),
        ifname=dict(type='str'),
        lldpadminstatus=dict(choices=['txOnly', 'rxOnly', 'txAndRx', 'disabled'], type='str'),
        manaddrtxenable=dict(type='bool'),
        portdesctxenable=dict(type='bool'),
        syscaptxenable=dict(type='bool'),
        sysdesctxenable=dict(type='bool'),
        sysnametxenable=dict(type='bool'),
        portvlantxenable=dict(type='bool'),
        protovlantxenable=dict(type='bool'),
        txprotocolvlanid=dict(type='int'),
        vlannametxenable=dict(type='bool'),
        txvlannameid=dict(type='int'),
        txinterval=dict(type='int'),
        protoidtxenable=dict(type='bool'),
        macphytxenable=dict(type='bool'),
        linkaggretxenable=dict(type='bool'),
        maxframetxenable=dict(type='bool'),
        eee=dict(type='bool'),
        dcbx=dict(type='bool'),
        state=dict(type='str', choices=['absent', 'present'], default='present'),
    )

    lldp_interface_obj = Lldp_interface(argument_spec)
    lldp_interface_obj.work()


if __name__ == '__main__':
    main()



from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class dummy_cursor_class():
    """Dummy class for returning an answer for SELECT VERSION()."""
    def __init__(self, output, ret_val_type='dict'):
        self.output = output
        self.ret_val_type = ret_val_type

    def execute(self, query):
        pass

    def fetchone(self):
        if self.ret_val_type == 'dict':
            return {'version': self.output}

        elif self.ret_val_type == 'list':
            return [self.output]



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for iosxr_ospf_interfaces
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = """
module: iosxr_ospf_interfaces
version_added: 1.2.0
short_description: OSPF Interfaces Resource Module.
description:
  - This module manages OSPF(v2/v3) configuration of interfaces on devices running Cisco IOS-XR.
author: Rohit Thakur (@rohitthakur2590)
notes:
  - Tested against IOS-XR 6.1.3
  - This module works with connection C(network_cli). See L(the IOS-XR Platform Options,../network/user_guide/platform_iosxr.html)
options:
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS-XR device
        by executing the command B(show running-config router ospf').
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description: A list of OSPF configuration for interfaces.
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Name/Identifier of the interface.
        type: str
        required: True
      type:
        description:
          - Type of the interface.
        type: str
        required: True
      address_family:
        description:
          - OSPF settings on the interfaces in address-family context.
        type: list
        elements: dict
        suboptions:
          afi:
            description:
              - Address Family Identifier (AFI) for OSPF settings on the interfaces.
            type: str
            choices: ['ipv4', 'ipv6']
            required: True
          processes:
            description:
              - Interfaces configuration for an OSPF process.
            type: list
            elements: dict
            suboptions:
              process_id:
                description:
                  - OSPF process tag.
                type: str
                required: True
              area:
                description: Specify the area-id
                type: dict
                suboptions:
                  area_id:
                    description:
                      - OSPF interfaces area ID as a decimal value. Please
                        refer vendor documentation of Valid values.
                      - OSPF interfaces area ID in IP address format(e.g.
                        A.B.C.D)
                    type: str
          apply_group_option:
            description: Specify configuration from a group
            type: dict
            suboptions:
              group_name:
                description: Specify the name of the group
                type: str
              operation:
                description: Specify the group config operation
                type: str
                choices: [add, remove, append]
          authentication:
            description: Enable authentication
            type: dict
            suboptions:
              message_digest:
                description: Use message-digest authentication
                type: dict
                suboptions:
                  keychain:
                    description: Specify keychain name
                    type: str
              null_auth:
                description: Use no authentication
                type: bool
          authentication_key:
            description: Specify authentication password (key)
            type: dict
            suboptions:
              password:
                description: The OSPFv2 password (key)
                type: str
              clear:
                description: Specifies an UNENCRYPTED password (key) will follow
                type: str
              encrypted:
                description: Specifies an ENCRYPTED password (key) will follow
                type: str
          bfd:
            description: Configure BFD parameters
            type: dict
            suboptions:
              fast_detect:
                description: Configure fast detection
                type: dict
                suboptions:
                  set:
                    description: Enable fast detection only
                    type: bool
                  strict_mode:
                    description: Hold down neighbor session until BFD session is up
                    type: bool
              minimum_interval:
                description: Hello interval in milli-seconds
                type: int
              multiplier:
                description: Detect multiplier
                type: int
          cost:
            description: Specify Interface cost
            type: int
          cost_fallback:
            description: Specify Cost when cumulative bandwidth goes below the theshold
            type: dict
            suboptions:
              cost:
                description: Specify cost w.r.t cummulative bandwidth
                type: int
              threshold:
                description: Specify threshold bandwidth when cost-fallback is applied
                type: int
          database_filter:
            description: Filter OSPF LSAs during synchronization and flooding
            type: dict
            suboptions:
              all_outgoing_lsa:
                description: Filter all outgoing LSA
                type: bool
          dead_interval:
            description: Specify interval after which a neighbor is declared dead
            type: int
          demand_circuit:
            description: Enable/Disable demand circuits
            type: bool
          fast_reroute:
            description: Specify IP Fast Reroute
            type: dict
            suboptions:
              disabled:
                description: Disable IP fast reroute
                type: bool
              per_link:
                description: Specify per-prefix computation
                type: dict
                suboptions:
                  information_type:
                    description: Specify per-link LFA exclusion or FRR LFA candidate information
                    type: str
                    choices: ["exclude", "lfa_candidate"]
                  use_candidate_only:
                    description: Enable/Disable backup selection from candidate-list only
                    type: bool
                  interface:
                    description: Specify Per-link LFA exclusion information
                    type: dict
                    suboptions:
                      bvi:
                        description: Specify Bridge-Group Virtual Interface
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: int
                      bundle_ether:
                        description: Specify Aggregated Ethernet interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: int
                      pos_int:
                        description: Specify Aggregated pos interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: int
                      fast_ethernet:
                        description: Specify FastEthernet/IEEE 802.3 interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: str
                      fiftygige:
                        description: Specify FiftyGigE/IEEE 802.3 interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: str
                      fortygige:
                        description: Specify FortyGigE/IEEE 802.3 interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: str
                      fourhundredgige:
                        description: Specify FourHundredGigE/IEEE 802.3 interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: str
                      gigabitethernet:
                        description: Specify GigabitEthernet/IEEE 802.3 interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: str
                      hundredgige:
                        description: Specify HundredGigE/IEEE 802.3 interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: str
                      mgmteth:
                        description: Specify MgmtEth/IEEE 802.3 interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: str
                      multilink:
                        description: Specify Multilink network interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: str
                      pw_ether:
                        description: Specify PWHE Ethernet Interface
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: int
                      pw_iw:
                        description: Specify PWHE VC11 IP Interworking Interface
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: int
                      srp:
                        description: Specify SRP interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: str
                      serial:
                        description: Specify Serial network interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: str
                      tengige:
                        description: Specify TenGigabitEthernet/IEEE 802.3 interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: str
                      twentyfivegige:
                        description: Specify TwentyFiveGigabitEthernet/IEEE 802.3 interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: str
                      twohundredgige:
                        description: Specify TwoHundredGigE/IEEE 802.3 interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: str
                      nve:
                        description: Specify Network Virtualization Endpoint Interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: int
                      tunnel_ip:
                        description: Specify GRE/IPinIP Tunnel Interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: int
                      tunnel_ipsec:
                        description: Specify IPSec Tunnel interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: int
                      tunnel_mte:
                        description: Specify MPLS Traffic Engineering P2MP Tunnel interface(s)
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: int
                      tunnel_mpls:
                        description: MPLS Transport Protocol Tunnel interface
                        type: list
                        elements: dict
                        suboptions:
                          name:
                            description: Specify the interface id
                            type: str
          flood_reduction:
            description: Enable/Disable flood reduction
            type: bool
          hello_interval:
            description: Specify Time between HELLO packets
            type: int
          link_down_fast_detect:
            description: Configure interface down parameters
            type: bool
          message_digest_key:
            description: Message digest authentication password (key)
            type: dict
            suboptions:
              id:
                description: Key ID
                type: int
                required: true
              md5:
                description: Use MD5 Algorithm
                type: dict
                required: true
                suboptions:
                  password:
                    description: The OSPFv2 password (key)
                    type: str
                  clear:
                    description: Specifies an UNENCRYPTED password (key) will follow
                    type: bool
                  encrypted:
                    description: Specifies an ENCRYPTED password (key) will follow
                    type: bool
          mpls_ldp_sync:
            description: Enable/Disable MPLS LDP Sync
            type: bool
          mtu_ignore:
            description: Enable/Disable ignoring of MTU in DBD packets
            type: bool
          network:
            description: Specify Network type
            type: str
            choices: ["broadcast", "non-broadcast", "point-to-multipoint", "point-to-point"]
          neighbors:
            description: Specify a neighbor routers
            type: list
            elements: dict
            suboptions:
              neighbor_id:
                description: Specify Neighbor address (name)
                type: str
              cost:
                description: Specify OSPF cost for point-to-multipoint neighbor
                type: int
              db_filter_all_out:
                description: Specify Filter OSPF LSA during synchronization and flooding for point-to-multipoint neighbor
                type: bool
              poll_interval:
                description: Specify OSPF dead-router polling interval
                type: int
              priority:
                description: Specify OSPF priority of non-broadcast neighbor
                type: int
          packet_size:
            description: Customize size of OSPF packets upto MTU
            type: int
          passive:
            description: Enable/Disable passive
            type: bool
          prefix_suppression:
            description: Suppress advertisement of the prefixes
            type: bool
          priority:
            description: Specify Router priority
            type: int
          retransmit_interval:
            description: Specify time between retransmitting lost link state advertisements
            type: int
          security_ttl:
            description: Enable security
            type: dict
            suboptions:
              set:
                description: Enable ttl security
                type: bool
              hops:
                description: Maximum number of IP hops allowed <1-254>
                type: int
          transmit_delay:
            description: Specify estimated time needed to send link-state update packet
            type: int
  state:
    description:
      - The state the configuration should be left in.
    type: str
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - gathered
      - parsed
      - rendered
    default: merged
"""
EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# RP/0/RP0/CPU0:anton#show running-config router ospf
# % No such configuration item(s)
#

- name: Merge provided OSPF interfaces configuration with the existing configuration
  cisco.iosxr.iosxr_ospf_interfaces:
    config:
      - name: GigabitEthernet0/0/0/0
        type: gigabitethernet
        address_family:
          - afi: ipv4
            processes:
              - process_id: "LAB3"
                area:
                  area_id: 0.0.0.3
            cost: 20
            authentication:
              message_digest:
                keychain: cisco
          - afi: ipv6
            processes:
              - process_id: "LAB3"
                area:
                  area_id: 0.0.0.2
            cost: 30
    state: merged

#
#
# ------------------------
# Module Execution Result
# ------------------------
#
#  "before": []
#
#  "commands": [
#        "router ospf LAB3 area 0.0.0.3 interface GigabitEthernet 0/0/0/0 cost 20",
#        "router ospf LAB3 area 0.0.0.3 interface GigabitEthernet 0/0/0/0 authentication message-digest",
#        "router ospf LAB3 area 0.0.0.3 interface GigabitEthernet 0/0/0/0 authentication message-digest keychain cisco",
#        "router ospfv3 LAB3 area 0.0.0.2 interface GigabitEthernet 0/0/0/0 cost 30"
#     ]
#
#  "after": [
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv4",
#                     "authentication": {
#                         "message_digest": {
#                             "keychain": "cisco"
#                         }
#                     },
#                     "cost": 20,
#                     "processes": [
#                         {
#                             "area": {
#                                 "area_id": "0.0.0.3"
#                             },
#                             "process_id": "LAB3"
#                         }
#                     ]
#                 },
#                 {
#                     "afi": "ipv6",
#                     "cost": 30,
#                     "processes": [
#                         {
#                             "area": {
#                                 "area_id": "0.0.0.2"
#                             },
#                             "process_id": "LAB3"
#                         }
#                     ]
#                 }
#             ],
#             "name": "GigabitEthernet0/0/0/0",
#             "type": "gigabitethernet"
#         }
#     ]
#
#
# ------------
# After state
# ------------
#
# RP/0/0/CPU0:an-iosxr-02#show running-config router ospf
# Thu Oct 23 06:00:57.217 UTC
# router ospf LAB
#  area 0.0.0.0
#  !
#  area 0.0.0.9
#  !
# !
# router ospf LAB1
#  area 0.0.0.1
#  !
#  area 0.0.0.3
#  !
# !
# router ospf LAB3
#  area 0.0.0.3
#   interface GigabitEthernet0/0/0/0
#    cost 20
#    authentication message-digest keychain cisco
#   !
#  !
# !
# router ospf ipv4
# !

# Using replaced
#
# ------------
# Before state
# ------------
#
#
# RP/0/0/CPU0:an-iosxr-02#show running-config router ospf
# Thu Oct 23 06:00:57.217 UTC
# router ospf LAB
#  area 0.0.0.0
#  !
#  area 0.0.0.9
#  !
# !
# router ospf LAB1
#  area 0.0.0.1
#  !
#  area 0.0.0.3
#  !
# !
# router ospf LAB3
#  area 0.0.0.3
#   interface GigabitEthernet0/0/0/0
#    cost 20
#    authentication message-digest keychain cisco
#   !
#  !
# !
# router ospf ipv4
# !

- name: Replace OSPF interfaces configuration
  cisco.iosxr.iosxr_ospf_interfaces:
    config:
      - name: GigabitEthernet0/0/0/0
        type: gigabitethernet
        address_family:
          - afi: ipv4
            processes:
              - process_id: "LAB3"
                area:
                  area_id: 0.0.0.3
            cost: 30
            authentication:
              message_digest:
                keychain: ciscoiosxr
          - afi: ipv6
            processes:
              - process_id: "LAB3"
                area:
                  area_id: 0.0.0.2
            cost: 30
    state: replaced

#
#
# ------------------------
# Module Execution Result
# ------------------------
#
#  "before": [
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv4",
#                     "authentication": {
#                         "message_digest": {
#                             "keychain": "cisco"
#                         }
#                     },
#                     "cost": 20,
#                     "processes": [
#                         {
#                             "area": {
#                                 "area_id": "0.0.0.3"
#                             },
#                             "process_id": "LAB3"
#                         }
#                     ]
#                 },
#                 {
#                     "afi": "ipv6",
#                     "cost": 30,
#                     "processes": [
#                         {
#                             "area": {
#                                 "area_id": "0.0.0.2"
#                             },
#                             "process_id": "LAB3"
#                         }
#                     ]
#                 }
#             ],
#             "name": "GigabitEthernet0/0/0/0",
#             "type": "gigabitethernet"
#         }
#     ]
#
#  "commands": [
#    "router ospf LAB3 area 0.0.0.3 interface GigabitEthernet 0/0/0/0 cost 30",
#    "router ospf LAB3 area 0.0.0.3 interface GigabitEthernet 0/0/0/0 authentication message-digest",
#    "router ospf LAB3 area 0.0.0.3 interface GigabitEthernet 0/0/0/0 authentication message-digest keychain ciscoiosxr"
#     ]
#
#  "after": [
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv4",
#                     "authentication": {
#                         "message_digest": {
#                             "keychain": "ciscoiosxr"
#                         }
#                     },
#                     "cost": 30,
#                     "processes": [
#                         {
#                             "area": {
#                                 "area_id": "0.0.0.3"
#                             },
#                             "process_id": "LAB3"
#                         }
#                     ]
#                 },
#                 {
#                     "afi": "ipv6",
#                     "cost": 30,
#                     "processes": [
#                         {
#                             "area": {
#                                 "area_id": "0.0.0.2"
#                             },
#                             "process_id": "LAB3"
#                         }
#                     ]
#                 }
#             ],
#             "name": "GigabitEthernet0/0/0/0",
#             "type": "gigabitethernet"
#         }
#     ]
#
#
# -----------
# After state
# -----------
#
# RP/0/0/CPU0:an-iosxr-02#show running-config router ospf
# Thu Oct 23 06:10:39.827 UTC
# router ospf LAB
#  area 0.0.0.0
#  !
#  area 0.0.0.9
#  !
# !
# router ospf LAB1
#  area 0.0.0.1
#  !
#  area 0.0.0.3
#  !
# !
# router ospf LAB3
#  area 0.0.0.3
#   interface GigabitEthernet0/0/0/0
#    cost 30
#    authentication message-digest keychain ciscoiosxr
#   !
#  !
# !
# router ospf ipv4
# !

- name: Override existing OSPF interfaces configuration
  cisco.iosxr.iosxr_ospf_interfaces:
    config:
      - name: GigabitEthernet0/0/0/1
        type: gigabitethernet
        address_family:
          - afi: ipv4
            processes:
              - process_id: "LAB1"
                area:
                  area_id: 0.0.0.3
            cost: 10
            authentication:
              message_digest:
                keychain: iosxr
    state: overridden

#
#
# ------------------------
# Module Execution Result
# ------------------------
#
#  "before": [
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv4",
#                     "authentication": {
#                         "message_digest": {
#                             "keychain": "ciscoiosxr"
#                         }
#                     },
#                     "cost": 30,
#                     "processes": [
#                         {
#                             "area": {
#                                 "area_id": "0.0.0.3"
#                             },
#                             "process_id": "LAB3"
#                         }
#                     ]
#                 },
#                 {
#                     "afi": "ipv6",
#                     "cost": 30,
#                     "processes": [
#                         {
#                             "area": {
#                                 "area_id": "0.0.0.2"
#                             },
#                             "process_id": "LAB3"
#                         }
#                     ]
#                 }
#             ],
#             "name": "GigabitEthernet0/0/0/0",
#             "type": "gigabitethernet"
#         }
#     ]
#
#  "commands": [
#         "no router ospf LAB3 area 0.0.0.3 interface GigabitEthernet 0/0/0/0",
#         "no router ospfv3 LAB3 area 0.0.0.2 interface GigabitEthernet 0/0/0/0",
#         "router ospf LAB1 area 0.0.0.3 interface GigabitEthernet 0/0/0/1 cost 10",
#         "router ospf LAB1 area 0.0.0.3 interface GigabitEthernet 0/0/0/1 authentication message-digest",
#         "router ospf LAB1 area 0.0.0.3 interface GigabitEthernet 0/0/0/1 authentication message-digest keychain iosxr"
#     ]
#
#  "after": [
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv4",
#                     "authentication": {
#                         "message_digest": {
#                             "keychain": "iosxr"
#                         }
#                     },
#                     "cost": 10,
#                     "processes": [
#                         {
#                             "area": {
#                                 "area_id": "0.0.0.3"
#                             },
#                             "process_id": "LAB1"
#                         }
#                     ]
#                 }
#             ],
#             "name": "GigabitEthernet0/0/0/1",
#             "type": "gigabitethernet"
#         }
#     ]
#
#
# -----------
# After state
# -----------
#
# RP/0/0/CPU0:an-iosxr-02#show running-config router ospf
# Thu Oct 23 06:28:15.025 UTC
# router ospf LAB
#  area 0.0.0.0
#  !
#  area 0.0.0.9
#  !
# !
# router ospf LAB1
#  area 0.0.0.1
#  !
#  area 0.0.0.3
#   interface GigabitEthernet0/0/0/1
#    cost 10
#    authentication message-digest keychain iosxr
#   !
#  !
# !
# router ospf LAB3
#  area 0.0.0.3
#  !
# !
# router ospf ipv4
# !

# Using deleted
#
# ------------
# Before state
# ------------
#
#
# RP/0/0/CPU0:an-iosxr-02#show running-config router ospf
# Thu Oct 23 06:28:15.025 UTC
# router ospf LAB
#  area 0.0.0.0
#  !
#  area 0.0.0.9
#  !
# !
# router ospf LAB1
#  area 0.0.0.1
#  !
#  area 0.0.0.3
#   interface GigabitEthernet0/0/0/1
#    cost 10
#    authentication message-digest keychain iosxr
#   !
#  !
# !
# router ospf LAB3
#  area 0.0.0.3
#  !
# !
# router ospf ipv4
# !

- name: Deleted existing OSPF interfaces from the device
  cisco.iosxr.iosxr_ospf_interfaces:
    config:
      - name: GigabitEthernet0/0/0/1
        type: gigabitethernet
    state: deleted

#
#
# ------------------------
# Module Execution Result
# ------------------------
#
#  "before": [
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv4",
#                     "authentication": {
#                         "message_digest": {
#                             "keychain": "iosxr"
#                         }
#                     },
#                     "cost": 10,
#                     "processes": [
#                         {
#                             "area": {
#                                 "area_id": "0.0.0.3"
#                             },
#                             "process_id": "LAB1"
#                         }
#                     ]
#                 }
#             ],
#             "name": "GigabitEthernet0/0/0/1",
#             "type": "gigabitethernet"
#         }
#     ],
#
#  "commands": [
#         "no router ospf LAB1 area 0.0.0.3 interface GigabitEthernet 0/0/0/1"
#     ]
#
#  "after": []
#
#
# -----------
# After state
# -----------
#
# RP/0/0/CPU0:an-iosxr-02#show running-config router ospf
# Thu Oct 23 06:34:38.319 UTC
# router ospf LAB
#  area 0.0.0.0
#  !
#  area 0.0.0.9
#  !
# !
# router ospf LAB1
#  area 0.0.0.1
#  !
#  area 0.0.0.3
#  !
# !
# router ospf LAB3
#  area 0.0.0.3
#  !
# !
# router ospf ipv4
# !

# Using parsed
# parsed.cfg
# ------------
# router ospf LAB
#  area 0.0.0.0
#  !
#  area 0.0.0.9
#  !
# !
# router ospf LAB1
#  area 0.0.0.1
#  !
#  area 0.0.0.3
#  !
# !
# router ospf LAB3
#  area 0.0.0.3
#   interface GigabitEthernet0/0/0/0
#    cost 20
#    authentication message-digest keychain cisco
#   !
#  !
# !
# router ospf ipv4
# !
- name: Parsed the device configuration to get output commands
  cisco.iosxr.iosxr_ospf_interfaces:
    running_config: "{{ lookup('file', './parsed.cfg') }}"
    state: parsed
#
#
# -------------------------
# Module Execution Result
# -------------------------
#
#
# "parsed": [
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv4",
#                     "authentication": {
#                         "message_digest": {
#                             "keychain": "cisco"
#                         }
#                     },
#                     "cost": 20,
#                     "processes": [
#                         {
#                             "area": {
#                                 "area_id": "0.0.0.3"
#                             },
#                             "process_id": "LAB3"
#                         }
#                     ]
#                 }
#             ],
#             "name": "GigabitEthernet0/0/0/0",
#             "type": "gigabitethernet"
#         }
#     ]
#
# Using rendered
#
#
- name: Render the commands for provided  configuration
  cisco.iosxr.iosxr_ospf_interfaces:
    config:
      - name: GigabitEthernet0/0/0/0
        type: gigabitethernet
        address_family:
          - afi: ipv4
            processes:
             - process_id: "LAB3"
               area:
                 area_id: 0.0.0.3
            cost: 20
            authentication:
              message_digest:
                keychain: cisco
          - afi: ipv6
            processes:
             - process_id: "LAB3"
               area:
                 area_id: 0.0.0.2
            cost: 30
    state: rendered

#
#
# -------------------------
# Module Execution Result
# -------------------------
#
#
# "rendered": [
#        "router ospf LAB3 area 0.0.0.3 interface GigabitEthernet 0/0/0/0 cost 20",
#        "router ospf LAB3 area 0.0.0.3 interface GigabitEthernet 0/0/0/0 authentication message-digest",
#        "router ospf LAB3 area 0.0.0.3 interface GigabitEthernet 0/0/0/0 authentication message-digest keychain cisco",
#        "router ospfv3 LAB3 area 0.0.0.2 interface GigabitEthernet 0/0/0/0 cost 30"
#     ]


# Using gathered
#
# Before state:
# -------------
#
# RP/0/0/CPU0:an-iosxr-02#show running-config router ospf
# Thu Oct 23 06:50:38.743 UTC
# router ospf LAB
#  area 0.0.0.0
#  !
#  area 0.0.0.9
#  !
# !
# router ospf LAB1
#  area 0.0.0.1
#  !
#  area 0.0.0.3
#  !
# !
# router ospf LAB3
#  area 0.0.0.3
#   interface GigabitEthernet0/0/0/0
#    cost 20
#    authentication message-digest keychain cisco
#   !
#  !
# !
# router ospf ipv4
# !


- name: Gather ospf_interfaces routes configuration
  cisco.iosxr.iosxr_ospf_interfaces:
    state: gathered
#
#
# -------------------------
# Module Execution Result
# -------------------------
#
#    "gathered": [
#         {
#             "address_family": [
#                 {
#                     "afi": "ipv4",
#                     "authentication": {
#                         "message_digest": {
#                             "keychain": "cisco"
#                         }
#                     },
#                     "cost": 20,
#                     "processes": [
#                         {
#                             "area": {
#                                 "area_id": "0.0.0.3"
#                             },
#                             "process_id": "LAB3"
#                         }
#                     ]
#                 },
#                 {
#                     "afi": "ipv6",
#                     "cost": 30,
#                     "processes": [
#                         {
#                             "area": {
#                                 "area_id": "0.0.0.2"
#                             },
#                             "process_id": "LAB3"
#                         }
#                     ]
#                 }
#             ],
#             "name": "GigabitEthernet0/0/0/0",
#             "type": "gigabitethernet"
#         }
#     ]
#
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.ospf_interfaces.ospf_interfaces import (
    Ospf_interfacesArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.ospf_interfaces.ospf_interfaces import (
    Ospf_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Ospf_interfacesArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Ospf_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()
